import datetime

from pyhafas import HafasClient
from pyhafas.profile import VSNProfile


def test_vsn_arrivals_request():
    client = HafasClient(VSNProfile())
    arrivals = client.arrivals(
        station="009033817",
        date=datetime.datetime.now(),
        max_trips=5
    )
    assert arrivals



import datetime

from pyhafas import HafasClient
from pyhafas.profile import KVBProfile


def test_kvb_journeys_request():
    client = HafasClient(KVBProfile())
    journeys = client.journeys(
        destination="900000687",
        origin="900000008",
        date=datetime.datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys

def test_kvb_journeys_local_request():
    client = HafasClient(KVBProfile())
    journeys = client.journeys(
        destination="900000001",
        origin="900000039",
        date=datetime.datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys



import datetime
import os

from pyhafas.profile import DBProfile
from pyhafas.types.fptf import Journey, Leg, Mode, Station, Stopover, Remark

from tests.types import PyTestHafasResponse


def test_db_journeys_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/journeys_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_journeys = [Journey(
        id='¶HKI¶T$A=1@O=Siegburg/Bonn@L=8005556@a=128@$A=1@O=Troisdorf@L=8000135@a=128@$202008061207$202008061212$S     19$$1$$$',
        date=datetime.date(2020, 8, 6),
        duration=datetime.timedelta(seconds=300),
        legs=[Leg(
            id='1|226393|0|80|6082020',
            origin=Station(
                id='8005556',
                lid='A=1@O=Siegburg/Bonn@X=7203029@Y=50793916@U=80@L=8005556@',
                name='Siegburg/Bonn',
                latitude=50.793916,
                longitude=7.203029),
            destination=Station(
                id='8000135',
                lid='A=1@O=Troisdorf@X=7150892@Y=50813926@U=80@L=8000135@',
                name='Troisdorf',
                latitude=50.813926,
                longitude=7.150892),
            departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 6, 12, 7)),
            arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 6, 12, 12)),
            mode=Mode.TRAIN,
            name='S 19',
            cancelled=False,
            distance=None,
            departure_delay=None,
            departure_platform='1',
            arrival_delay=datetime.timedelta(0),
            arrival_platform='1',
            stopovers=[Stopover(
                stop=Station(
                    id='8005556',
                    lid='A=1@O=Siegburg/Bonn@X=7203029@Y=50793916@U=80@L=8005556@',
                    name='Siegburg/Bonn',
                    latitude=50.793916,
                    longitude=7.203029),
                cancelled=False,
                arrival=None,
                arrival_delay=None,
                arrival_platform=None,
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 6, 12, 7)),
                departure_delay=None,
                departure_platform='1',
                remarks=[]),
                Stopover(
                    stop=Station(
                        id='8000135',
                        lid='A=1@O=Troisdorf@X=7150892@Y=50813926@U=80@L=8000135@',
                        name='Troisdorf',
                        latitude=50.813926,
                        longitude=7.150892),
                    cancelled=False,
                    arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 6, 12, 12)),
                    arrival_delay=datetime.timedelta(0),
                    arrival_platform='1',
                    departure=None,
                    departure_delay=None,
                    departure_platform=None,
                    remarks=[],
                )
            ],
            remarks=[
                Remark(
                    remark_type='A',
                    code='FB',
                    subject=None,
                    text='Fahrradmitnahme begrenzt möglich',
                    priority=260,
                    trip_id=None
                ),
                Remark(
                    remark_type='A',
                    code='K2',
                    subject=None,
                    text='nur 2. Klasse',
                    priority=300,
                    trip_id=None
                ),
                Remark(
                    remark_type='A',
                    code='EH',
                    subject=None,
                    text='Fahrzeuggebundene Einstiegshilfe vorhanden',
                    priority=560,
                    trip_id=None
                )
            ]
        )],
    )]
    assert DBProfile().parse_journeys_request(hafas_response) == correct_journeys



import datetime
from typing import Dict, List

from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.requests.journeys import \
    JourneysRequestInterface
from pyhafas.types.fptf import Journey, Station, Leg
from pyhafas.types.hafas_response import HafasResponse


class BaseJourneysRequest(JourneysRequestInterface):
    def format_journeys_request(
            self: ProfileInterface,
            origin: Station,
            destination: Station,
            via: List[Station],
            date: datetime.datetime,
            min_change_time: int,
            max_changes: int,
            products: Dict[str, bool],
            max_journeys: int
    ) -> dict:
        """
        Creates the HaFAS request body for a journeys request

        :param origin: Origin station
        :param destination: Destionation station
        :param via: Via stations, maybe empty list)
        :param date: Date and time to search journeys for
        :param min_change_time: Minimum transfer/change time at each station
        :param max_changes: Maximum number of changes
        :param products: Allowed products (a product is a mean of transport like ICE,IC)
        :param max_journeys: Maximum number of returned journeys
        :return: Request body for HaFAS
        """
        # TODO: find out, what commented-out values mean and implement options
        return {
            'req': {
                'arrLocL': [{
                    'type': 'S',
                    'lid': 'A=1@L={}@'.format(destination.id)
                }],
                'viaLocL': [{
                    'loc': {
                        'type': 'S',
                        'lid': 'A=1@L={}@'.format(via_station.id)
                    }
                } for via_station in via],
                'depLocL': [{
                    'type': 'S',
                    'lid': 'A=1@L={}@'.format(origin.id)
                }],
                'outDate': date.strftime("%Y%m%d"),
                'outTime': date.strftime("%H%M%S"),
                'jnyFltrL': [
                    self.format_products_filter(products)
                ],
                'minChgTime': min_change_time,
                'maxChg': max_changes,
                'numF': max_journeys,
                # 'getPasslist': False,
                # 'gisFltrL': [],
                # 'getTariff': False,
                # 'ushrp': True,
                # 'getPT': True,
                # 'getIV': False,
                # 'getPolyline': False,
                # 'outFrwd': True,
                # 'trfReq': {
                #    'jnyCl': 2,
                #    'cType': 'PK',
                #    'tvlrProf': [{
                #        'type': 'E',
                #        'redtnCard': 4
                #    }]
                # }
            },
            # 'cfg': {
            #    'polyEnc': 'GPA',
            #    'rtMode': 'HYBRID'
            # },
            'meth': 'TripSearch'
        }

    def format_search_from_leg_request(
            self: ProfileInterface,
            origin: Leg,
            destination: Station,
            via: List[Station],
            min_change_time: int,
            max_changes: int,
            products: Dict[str, bool],
    ) -> dict:
        """
        Creates the HaFAS request body for a journeys request

        :param origin: Origin leg
        :param destination: Destionation station
        :param via: Via stations, maybe empty list)
        :param min_change_time: Minimum transfer/change time at each station
        :param max_changes: Maximum number of changes
        :param products: Allowed products (a product is a mean of transport like ICE,IC)
        :return: Request body for HaFAS
        """
        return {
            'req': {
                'arrLocL': [{
                    'lid': 'A=1@L={}@'.format(destination.id)
                }],
                'viaLocL': [{
                    'loc': {
                        'lid': 'A=1@L={}@'.format(via_station.id)
                    }
                } for via_station in via],
                'locData': {
                    'loc': {
                        'lid': 'A=1@L={}@'.format(origin.origin.id)
                    },
                    'type': 'DEP',
                    'date': origin.departure.strftime("%Y%m%d"),
                    'time': origin.departure.strftime("%H%M%S")
                },
                'jnyFltrL': [
                    self.format_products_filter(products)
                ],
                'minChgTime': min_change_time,
                'maxChg': max_changes,
                'jid': origin.id,
                'sotMode': 'JI'
            },
            'meth': 'SearchOnTrip'
        }

    def parse_journeys_request(
            self: ProfileInterface,
            data: HafasResponse) -> List[Journey]:
        """
        Parses the HaFAS response for a journeys request

        :param data: Formatted HaFAS response
        :return: List of Journey objects
        """
        journeys = []

        for jny in data.res['outConL']:
            # TODO: Add more data
            date = self.parse_date(jny['date'])
            journeys.append(
                Journey(
                    jny['ctxRecon'], date=date, duration=self.parse_timedelta(
                        jny['dur']), legs=self.parse_legs(
                        jny, data.common, date)))
        return journeys



from .interfaces import ProfileInterface  # isort:skip
from .base import BaseProfile
from .db import DBProfile
from .vsn import VSNProfile
from .rkrp import RKRPProfile
from .nasa import NASAProfile
from .kvb import KVBProfile
from .nvv import NVVProfile
from .vvv import VVVProfile



import datetime

from pyhafas import HafasClient
from pyhafas.profile import NVVProfile


def test_nvv_journeys_request():
    client = HafasClient(NVVProfile())
    journeys = client.journeys(
        destination="2202508",
        origin="2200001",
        date=datetime.datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys



from pyhafas import HafasClient
from pyhafas.profile import KVBProfile


def test_kvb_locations_request():
    client = HafasClient(KVBProfile())
    locations = client.locations(term="Bonn Hbf")
    assert len(locations) >= 1



from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.requests.journey import JourneyRequestInterface
from pyhafas.types.fptf import Journey
from pyhafas.types.hafas_response import HafasResponse


class BaseJourneyRequest(JourneyRequestInterface):
    def format_journey_request(
            self: ProfileInterface,
            journey: Journey) -> dict:
        """
        Creates the HaFAS request body for a journey request

        :param journey: Id of the journey (ctxRecon)
        :return: Request body for HaFAS
        """
        return {
            'req': {
                'ctxRecon': journey.id
            },
            'meth': 'Reconstruction'
        }

    def parse_journey_request(
            self: ProfileInterface,
            data: HafasResponse) -> Journey:
        """
        Parses the HaFAS response for a journey request

        :param data: Formatted HaFAS response
        :return: List of Journey objects
        """
        date = self.parse_date(data.res['outConL'][0]['date'])
        return Journey(
            data.res['outConL'][0]['ctxRecon'],
            date=date,
            duration=self.parse_timedelta(
                data.res['outConL'][0]['dur']),
            legs=self.parse_legs(
                data.res['outConL'][0],
                data.common,
                date))



import datetime
import os

from pyhafas.profile import DBProfile
from pyhafas.types.fptf import Leg, Mode, Station, Stopover, Remark

from tests.types import PyTestHafasResponse


def test_db_trips_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/trip_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_trip = Leg(
        id='1|227083|0|80|5082020',
        origin=Station(
            id='8002753',
            lid='A=1@O=Hennef(Sieg)@X=7284588@Y=50773331@U=80@L=8002753@',
            name='Hennef(Sieg)',
            latitude=50.773331,
            longitude=7.284588),
        destination=Station(
            id='8000084',
            lid='A=1@O=Düren@X=6482454@Y=50809513@U=80@L=8000084@',
            name='Düren',
            latitude=50.809513,
            longitude=6.482454
        ),
        departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 2)),
        arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 17)),
        mode=Mode.TRAIN,
        name='S 19',
        cancelled=False,
        distance=None,
        departure_delay=None,
        departure_platform='2',
        arrival_delay=None,
        arrival_platform='5',
        stopovers=[
            Stopover(
                stop=Station(
                    id='8002753',
                    lid='A=1@O=Hennef(Sieg)@X=7284588@Y=50773331@U=80@L=8002753@',
                    name='Hennef(Sieg)',
                    latitude=50.773331,
                    longitude=7.284588
                ),
                cancelled=False,
                arrival=None,
                arrival_delay=None,
                arrival_platform=None,
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 2)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8005556',
                    lid='A=1@O=Siegburg/Bonn@X=7203029@Y=50793916@U=80@L=8005556@',
                    name='Siegburg/Bonn',
                    latitude=50.793916,
                    longitude=7.203029
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 6)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 7)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8000135',
                    lid='A=1@O=Troisdorf@X=7150892@Y=50813926@U=80@L=8000135@',
                    name='Troisdorf',
                    latitude=50.813926,
                    longitude=7.150892
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 12)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 13)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8005629',
                    lid='A=1@O=Spich@X=7114917@Y=50826727@U=80@L=8005629@',
                    name='Spich',
                    latitude=50.826727,
                    longitude=7.114917
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 15)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 16)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8004873',
                    lid='A=1@O=Porz-Wahn@X=7079266@Y=50858135@U=80@L=8004873@',
                    name='Porz-Wahn',
                    latitude=50.858135,
                    longitude=7.079266
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 19)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 19)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003330',
                    lid='A=1@O=Köln/Bonn Flughafen@X=7119304@Y=50878900@U=80@L=8003330@',
                    name='Köln/Bonn Flughafen',
                    latitude=50.8789,
                    longitude=7.119304
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 23)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 24)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003358',
                    lid='A=1@O=Köln Frankfurter Straße@X=7051264@Y=50915217@U=80@L=8003358@',
                    name='Köln Frankfurter Straße',
                    latitude=50.915217,
                    longitude=7.051264
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 29)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 29)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003320',
                    lid='A=1@O=Köln Trimbornstr@X=6996736@Y=50935856@U=80@L=8003320@',
                    name='Köln Trimbornstr',
                    latitude=50.935856,
                    longitude=6.996736),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 34)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 34)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8083368',
                    lid='A=1@O=Köln Messe/Deutz Gl. 9-10@X=6974640@Y=50941303@U=80@L=8083368@',
                    name='Köln Messe/Deutz Gl. 9-10',
                    latitude=50.941303,
                    longitude=6.97464),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 36)),
                arrival_delay=None,
                arrival_platform='10',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 37)),
                departure_delay=None,
                departure_platform='10',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8000207',
                    lid='A=1@O=Köln Hbf@X=6958730@Y=50943029@U=80@L=8000207@',
                    name='Köln Hbf',
                    latitude=50.943029,
                    longitude=6.95873
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 39)),
                arrival_delay=None,
                arrival_platform='11 B-C',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 40)),
                departure_delay=None,
                departure_platform='11 B-C',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003392',
                    lid='A=1@O=Köln Hansaring@X=6952563@Y=50949133@U=80@L=8003392@',
                    name='Köln Hansaring',
                    latitude=50.949133,
                    longitude=6.952563
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 42)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 42)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8000208',
                    lid='A=1@O=Köln-Ehrenfeld@X=6917280@Y=50951533@U=80@L=8000208@',
                    name='Köln-Ehrenfeld',
                    latitude=50.951533,
                    longitude=6.91728),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 45)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 46)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003375',
                    lid='A=1@O=Köln-Müngersdorf Technologiepark@X=6888200@Y=50948396@U=80@L=8003375@',
                    name='Köln-Müngersdorf Technologiepark',
                    latitude=50.948396,
                    longitude=6.8882
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 48)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 49)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003732',
                    lid='A=1@O=Lövenich@X=6834436@Y=50942930@U=80@L=8003732@',
                    name='Lövenich',
                    latitude=50.94293,
                    longitude=6.834436
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 51)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 52)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003383',
                    lid='A=1@O=Köln-Weiden West@X=6815136@Y=50940899@U=80@L=8003383@',
                    name='Köln-Weiden West',
                    latitude=50.940899,
                    longitude=6.815136
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 53)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 54)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8002389',
                    lid='A=1@O=Frechen-Königsdorf@X=6777849@Y=50936503@U=80@L=8002389@',
                    name='Frechen-Königsdorf',
                    latitude=50.936503,
                    longitude=6.777849
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 56)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 12, 57)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8000178',
                    lid='A=1@O=Horrem@X=6713495@Y=50916250@U=80@L=8000178@',
                    name='Horrem',
                    latitude=50.91625,
                    longitude=6.713495
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 0)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 1)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8005575',
                    lid='A=1@O=Sindorf@X=6681107@Y=50903711@U=80@L=8005575@',
                    name='Sindorf',
                    latitude=50.903711,
                    longitude=6.681107
                ), cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 3)),
                arrival_delay=None,
                arrival_platform='1',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 3)),
                departure_delay=None,
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8001264',
                    lid='A=1@O=Buir@X=6574513@Y=50862405@U=80@L=8001264@',
                    name='Buir',
                    latitude=50.862405,
                    longitude=6.574513
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 9)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 10)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8003990',
                    lid='A=1@O=Merzenich@X=6518051@Y=50840031@U=80@L=8003990@',
                    name='Merzenich',
                    latitude=50.840031,
                    longitude=6.518051
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 13)),
                arrival_delay=None,
                arrival_platform='2',
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 13)),
                departure_delay=None,
                departure_platform='2',
                remarks=[],
            ),
            Stopover(
                stop=Station(
                    id='8000084',
                    lid='A=1@O=Düren@X=6482454@Y=50809513@U=80@L=8000084@',
                    name='Düren',
                    latitude=50.809513,
                    longitude=6.482454
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 13, 17)),
                arrival_delay=None,
                arrival_platform='5',
                departure=None,
                departure_delay=None,
                departure_platform=None,
                remarks=[],
            )
        ],
        remarks=[
            Remark(
                remark_type='A',
                code='FB',
                subject=None,
                text='Fahrradmitnahme begrenzt möglich',
                priority=260,
                trip_id=None
            ),
            Remark(
                remark_type='A',
                code='K2',
                subject=None,
                text='nur 2. Klasse',
                priority=300,
                trip_id=None
            ),
            Remark(
                remark_type='A',
                code='EH',
                subject=None,
                text='Fahrzeuggebundene Einstiegshilfe vorhanden',
                priority=560,
                trip_id=None
            )
        ]
    )
    assert DBProfile().parse_trip_request(hafas_response) == correct_trip



import pytz

from pyhafas.profile.base import BaseProfile
from pyhafas.profile.vvv.requests.journey import VVVJourneyRequest
from pyhafas.profile.vvv.requests.journeys import VVVJourneysRequest


class VVVProfile(VVVJourneyRequest, VVVJourneysRequest, BaseProfile):
    """
    Profile of the HaFAS of Verkehrsverbund Vorarlberg (VVV)
    https://de.wikipedia.org/wiki/Verkehrsverbund_Vorarlberg
    """

    baseUrl = "https://fahrplan.vmobil.at/bin/mgate.exe"

    addMicMac = True
    salt = '6633673735743766726667323938336A'

    locale = 'at-DE'
    timezone = pytz.timezone('Europe/Vienna')

    requestBody = {
        "client": {
            "id": "VAO",
            "l": "vs_vvv",
            "name": "webapp",
            "type": "WEB",
            "v": "20230901"
        },
        "ext": "VAO.20",
        "ver": "1.59",
        "lang": "de",
        "auth": {
            "type": "AID",
            "aid": "wf7mcf9bv3nv8g5f"
        }
    }

    availableProducts = {
        'train-and-s-bahn': [1, 2],  # Bahn & S-Bahn
        'u-bahn': [4],  # U-Bahn
        'tram': [16],  # Straßenbahn
        'long-distance-bus': [32],  # Fernbus
        'regional-bus': [64],  # Regionalbus
        'city-bus': [128],  # Stadtbus
        'aerial-lift': [256],  # Seil-/Zahnradbahn
        'ferry': [512],  # Schiff
        'on-call': [1024],  # Anrufsammeltaxi
        'other-bus': [2048]  # sonstige Busse
    }

    defaultProducts = [
        'train-and-s-bahn',
        'u-bahn',
        'tram',
        'long-distance-bus',
        'regional-bus',
        'city-bus',
        'aerial-lift',
        'ferry',
        'on-call',
        'other-bus'
    ]



from enum import Enum


class ErrorCodesMappingInterface(Enum):
    """
    Mapping of the HaFAS error code to the exception class

    `default` defines the error when the error code cannot be found in the mapping
    """
    default: Exception



from pyhafas import HafasClient
from pyhafas.profile import NVVProfile
from pyhafas.types.fptf import Journey


def test_nvv_journey_request():
    client = HafasClient(NVVProfile())
    journey = client.journey(
        journey="¶HKI¶T$A=1@O=Rotenburg (Fulda) Bahnhof@L=2202508@a=128@$A=1@O=Kassel Bahnhof Wilhelmshöhe@L=2200007@a=128@$202312232105$202312232142$ RB5$$1$$$$$$¶KC¶#VE#2#CF#100#CA#0#CM#0#SICT#0#AM#49#AM2#0#RT#7#¶KRCC¶#VE#1#¶PC¶eyJncnBJZCI6IkdST1VQX1BUIn0="
    )
    assert isinstance(journey, Journey)



import abc

from pyhafas.types.fptf import Station


class ParseLidHelperInterface(abc.ABC):
    @abc.abstractmethod
    def parse_lid(self, lid: str) -> dict:
        """
        Converts the LID given by HaFAS. Splits the LID in multiple elements

        :param lid: Location identifier (given by HaFAS)
        :return: Dict of the elements of the dict
        """
        pass

    @abc.abstractmethod
    def parse_lid_to_station(
            self,
            lid: str,
            name: str = "",
            latitude: float = 0,
            longitude: float = 0) -> Station:
        """
        Parses the LID given by HaFAS to a station object

        :param lid: Location identifier (given by HaFAS)
        :param name: Station name (optional)
        :param latitude: Latitude of the station (optional)
        :param longitude: Longitude of the station (optional)
        :return: Parsed LID as station object
        """
        pass



import datetime
import os

from pyhafas.profile import DBProfile
from pyhafas.types.fptf import Journey, Leg, Mode, Station, Stopover, Remark

from tests.types import PyTestHafasResponse


def test_db_journey_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/journey_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_journey = Journey(
        id='¶HKI¶T$A=1@O=Siegburg/Bonn@L=8005556@a=128@$A=1@O=Troisdorf@L=8000135@a=128@$202008081507$202008081512$S     19$$1$$$',
        date=datetime.date(2020, 8, 8),
        duration=datetime.timedelta(seconds=300),
        legs=[Leg(
            id='1|227361|0|80|8082020',
            origin=Station(
                id='8005556',
                lid='A=1@O=Siegburg/Bonn@X=7203029@Y=50793916@U=80@L=8005556@',
                name='Siegburg/Bonn',
                latitude=50.793916,
                longitude=7.203029),
            destination=Station(
                id='8000135',
                lid='A=1@O=Troisdorf@X=7150892@Y=50813926@U=80@L=8000135@',
                name='Troisdorf',
                latitude=50.813926,
                longitude=7.150892),
            departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 8, 15, 7)),
            arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 8, 15, 12)),
            mode=Mode.TRAIN,
            name='S 19',
            cancelled=False, distance=None,
            departure_delay=datetime.timedelta(seconds=240),
            departure_platform='1',
            arrival_delay=datetime.timedelta(seconds=240),
            arrival_platform='1',
            stopovers=[Stopover(
                stop=Station(
                    id='8005556',
                    lid='A=1@O=Siegburg/Bonn@X=7203029@Y=50793916@U=80@L=8005556@',
                    name='Siegburg/Bonn',
                    latitude=50.793916,
                    longitude=7.203029
                ),
                cancelled=False,
                arrival=None,
                arrival_delay=None,
                arrival_platform=None,
                departure=DBProfile().timezone.localize(datetime.datetime(2020, 8, 8, 15, 7)),
                departure_delay=datetime.timedelta(seconds=240),
                departure_platform='1',
                remarks=[],
            ), Stopover(
                stop=Station(
                    id='8000135',
                    lid='A=1@O=Troisdorf@X=7150892@Y=50813926@U=80@L=8000135@',
                    name='Troisdorf',
                    latitude=50.813926,
                    longitude=7.150892
                ),
                cancelled=False,
                arrival=DBProfile().timezone.localize(datetime.datetime(2020, 8, 8, 15, 12)),
                arrival_delay=datetime.timedelta(seconds=240),
                arrival_platform='1',
                departure=None,
                departure_delay=None,
                departure_platform=None,
                remarks=[],
            ),
        ],
        remarks=[
            Remark(
                remark_type='A',
                code='FB',
                subject=None,
                text='Fahrradmitnahme begrenzt möglich',
                priority=260,
                trip_id=None
            ),
            Remark(
                remark_type='A',
                code='K2',
                subject=None,
                text='nur 2. Klasse',
                priority=300,
                trip_id=None
            ),
            Remark(
                remark_type='A',
                code='EH',
                subject=None,
                text='Fahrzeuggebundene Einstiegshilfe vorhanden',
                priority=560,
                trip_id=None
            )
        ])]
    )
    assert DBProfile().parse_journey_request(hafas_response) == correct_journey



import abc

from pyhafas.types.fptf import Remark


class ParseRemarkHelperInterface(abc.ABC):
    @abc.abstractmethod
    def parse_remark(
            self,
            remark: dict,
            common: dict) -> Remark:
        """
        Parses Remark HaFAS returns into Remark object

        :param remark: Remark object given back by HaFAS
        :param common:  Common object given back by HaFAS
        :return: Parsed Remark object
        """
        pass



from pyhafas import HafasClient
from pyhafas.profile import NASAProfile


# Test Location request of NASA by searching for 'Universtitätsbibliothek, Magdeburg', assert that at least one result is returned
def test_locations_request_nasa():
    client = HafasClient(NASAProfile())
    result = client.locations('Universtitätsbibliothek, Magdeburg')
    assert len(result) > 0



import datetime

from pyhafas import HafasClient
from pyhafas.profile import DBProfile


def test_db_arrivals_request():
    client = HafasClient(DBProfile())
    arrivals = client.arrivals(
        station="8011160",
        date=datetime.datetime.now(),
        max_trips=2
    )
    assert len(arrivals) <= 2



import abc
import datetime
from typing import List

from pyhafas.types.fptf import Leg


class ParseLegHelperInterface(abc.ABC):
    @abc.abstractmethod
    def parse_leg(
            self,
            journey: dict,
            common: dict,
            departure: dict,
            arrival: dict,
            date: datetime.date,
            jny_type: str = "JNY",
            gis=None) -> Leg:
        """
        Parses Leg HaFAS returns into Leg object

        :param journey: Journey object given back by HaFAS (Data of the Leg to parse)
        :param common:  Common object given back by HaFAS
        :param departure: Departure object given back by HaFAS
        :param arrival: Arrival object given back by HaFAS
        :param date: Parsed date of Journey (Departing date)
        :param jny_type: HaFAS Journey type
        :param gis: GIS object given back by HaFAS.
        :return: Parsed Leg object
        """
        pass

    @abc.abstractmethod
    def parse_legs(
            self,
            jny: dict,
            common: dict,
            date: datetime.date) -> List[Leg]:
        """
        Parses Legs (when multiple available)

        :param jny: Journeys object returned by HaFAS
        :param common: Common object returned by HaFAS
        :param date: Parsed date of Journey (Departing date)
        :return: Parsed List of Leg objects
        """
        pass



from enum import Enum


class StationBoardRequestType(Enum):
    """
    Mapping of StationBoard request from client to HaFAS
    """
    DEPARTURE = 'DEP'
    ARRIVAL = 'ARR'

    def __repr__(self):
        return '<%s.%s>' % (self.__class__.__name__, self.name)



from .client import HafasClient
from .types.exceptions import (AccessDeniedError, AuthenticationError,
                               GeneralHafasError,
                               JourneysArrivalDepartureTooNearError,
                               JourneysTooManyTrainsError,
                               LocationNotFoundError, TripDataNotFoundError)



import datetime

from pyhafas import HafasClient
from pyhafas.profile import NVVProfile


def test_kvb_departures_request():
    client = HafasClient(NVVProfile())
    departures = client.departures(
        station="2200001",
        date=datetime.datetime.now(),
        max_trips=5
    )
    assert departures



import datetime

from pyhafas import HafasClient
from pyhafas.profile import VVVProfile


def test_vvv_journeys_request():
    client = HafasClient(VVVProfile())
    journeys = client.journeys(
        destination="900006625",
        origin="480074302",
        date=datetime.datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys



from abc import ABC
from typing import Dict, List

import pytz

from pyhafas.profile.interfaces.helper.date_time import DateTimeHelperInterface
from pyhafas.profile.interfaces.helper.format_products_filter import \
    FormatProductsFilterHelperInterface
from pyhafas.profile.interfaces.helper.parse_leg import ParseLegHelperInterface
from pyhafas.profile.interfaces.helper.parse_lid import ParseLidHelperInterface
from pyhafas.profile.interfaces.helper.parse_remark import ParseRemarkHelperInterface
from pyhafas.profile.interfaces.helper.request import RequestHelperInterface
from pyhafas.profile.interfaces.requests.journey import JourneyRequestInterface
from pyhafas.profile.interfaces.requests.journeys import \
    JourneysRequestInterface
from pyhafas.profile.interfaces.requests.location import \
    LocationRequestInterface
from pyhafas.profile.interfaces.requests.station_board import \
    StationBoardRequestInterface
from pyhafas.profile.interfaces.requests.trip import TripRequestInterface


class ProfileInterface(
        RequestHelperInterface,
        FormatProductsFilterHelperInterface,
        ParseLidHelperInterface,
        DateTimeHelperInterface,
        ParseLegHelperInterface,
        ParseRemarkHelperInterface,
        LocationRequestInterface,
        JourneyRequestInterface,
        JourneysRequestInterface,
        StationBoardRequestInterface,
        TripRequestInterface,
        ABC):
    """
    The profile interface is the abstract class of a profile.

    It inherits all interfaces, so it has all methods a normal profile has available as abstract methods.
    Therefore it can be used as type hint for `self` in methods which are inherited by profiles.
    """
    baseUrl: str
    """Complete http(s) URL to mgate.exe of the HaFAS deployment. Other endpoints are (currently) incompatible with pyHaFAS"""
    addMicMac: bool
    """Whether the mic-mac authentication method should be activated. Exclusive with `addChecksum`"""
    addChecksum: bool
    """Whether the checksum authentication method should be activated. Exclusive with `addMicMac`"""
    salt: str
    """(required if `addMicMac` or `addChecksum` is true). The salt for calculating the checksum or mic-mac"""

    locale: str
    """(used in future) Locale used for i18n. Should be an IETF Language-Region Tag

    Examples: https://tools.ietf.org/html/bcp47#appendix-A
    """
    timezone: pytz.tzinfo.tzinfo
    """Timezone HaFAS lives in. Should be a `pytz` `timezone` object"""

    requestBody: dict
    """Static part of the request body sent to HaFAS. Normally contains informations about the client and another authentication"""

    availableProducts: Dict[str, List[int]]
    """Should contain all products available in HaFAS. The key is the name the end-user will use, the value is a list of bitmasks (numbers) needed for the product. In most cases, this is only one number. This bitmasks will be add up to generate the final bitmask."""
    defaultProducts: List[str]
    """List of products (item must be a key in `availableProducts`) which should be activated by default"""

    defaultUserAgent: str
    """(optional) User-Agent in header when connecting to HaFAS. Defaults to pyhafas. A good option would be the app ones. Can be overwritten by the user."""
    userAgent: str
    """(optional) Do not change, unless you know what you're doing. Disallows the user to change the user agent. For usage in internal code to get the user-agent which should be used."""



import datetime

from pyhafas import HafasClient
from pyhafas.profile import RKRPProfile

client = HafasClient(RKRPProfile(), debug=True)

# Human input strings
id_nordhavn = 'Nordhavn St.'
id_kongebakken9 = 'Kongebakken 9, 2765 Smørum'

# Resolve human string into Location-ID
lids_kongebakken9 = client.locations(id_kongebakken9, rtype='ALL')
lid_kongebakken9 = lids_kongebakken9[0].lid  # First is good-enough
assert('Kongebakken 9, 2765 Smørum, Egedal Kommune' in lid_kongebakken9)

lids_nordhavn = client.locations(id_nordhavn, rtype='ALL')
lid_nordhavn = lids_nordhavn[0].lid  # First is good-enough
assert('Nordhavn St' in lid_nordhavn)

#print(client.departures(
#    station=lid_nordhavn,
#    date=datetime.datetime.now(),
#    max_trips=5
#))

#print(client.arrivals(
#    station=lid_nordhavn,
#    date=datetime.datetime.now(),
#    max_trips=5
#))

possibilities = client.journeys(
    origin          = lid_nordhavn,
    destination     = lid_kongebakken9,
    date            = datetime.datetime.now(),
    min_change_time = 0,
    max_changes     = -1
)


for p in possibilities:
    print('--------')
    for l in p.legs:
        print(l.departure, l.origin.name, "--> " + str(l.name) + " -->" )

    l = p.legs[-1]
    print(l.arrival, l.destination.name)
    print("Journey duration: ", p.duration)



import datetime
import os

from pyhafas.profile.vsn import VSNProfile
from pyhafas.types.fptf import (Journey, Leg, Mode, Station, StationBoardLeg,
                                Stopover, Remark)

from tests.types import PyTestHafasResponse


def test_vsn_journeys_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/journeys_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_journeys = [
        Journey(
            id='¶HKI¶T$A=1@O=Göttingen@L=8000128@a=128@$A=1@O=Lenglern@L=8003644@a=128@$202008090710$202008090719$    RB85$$1$$$',
            date=datetime.date(
                2020,
                8,
                9),
            duration=datetime.timedelta(
                seconds=540),
            legs=[
                Leg(
                    id='1|147532|0|80|9082020',
                    origin=Station(
                        id='8000128',
                        lid='A=1@O=Göttingen@X=9926069@Y=51536812@U=80@L=8000128@',
                        name='Göttingen',
                        latitude=51.536812,
                        longitude=9.926069
                    ),
                    destination=Station(
                        id='8003644',
                        lid='A=1@O=Lenglern@X=9871199@Y=51588428@U=80@L=8003644@',
                        name='Lenglern',
                        latitude=51.588428,
                        longitude=9.871199
                    ),
                    departure=VSNProfile().timezone.localize(datetime.datetime(
                        2020,
                        8,
                        9,
                        7,
                        10
                    )),
                    arrival=VSNProfile().timezone.localize(datetime.datetime(
                        2020,
                        8,
                        9,
                        7,
                        19
                    )),
                    mode=Mode.TRAIN,
                    name='RB85',
                    cancelled=False,
                    distance=None,
                    departure_delay=datetime.timedelta(seconds=0),
                    departure_platform='4',
                    arrival_delay=datetime.timedelta(seconds=60),
                    arrival_platform='1',
                    stopovers=[
                        Stopover(
                            stop=Station(
                                id='8000128',
                                lid='A=1@O=Göttingen@X=9926069@Y=51536812@U=80@L=8000128@',
                                name='Göttingen',
                                latitude=51.536812,
                                longitude=9.926069
                            ),
                            cancelled=False,
                            arrival=None,
                            arrival_delay=None,
                            arrival_platform=None,
                            departure=VSNProfile().timezone.localize(datetime.datetime(
                                2020,
                                8,
                                9,
                                7,
                                10)),
                            departure_delay=datetime.timedelta(seconds=0),
                            departure_platform=None
                        ),
                        Stopover(
                            stop=Station(
                                id='8003644',
                                lid='A=1@O=Lenglern@X=9871199@Y=51588428@U=80@L=8003644@',
                                name='Lenglern',
                                latitude=51.588428,
                                longitude=9.871199
                            ),
                            cancelled=False,
                            arrival=VSNProfile().timezone.localize(datetime.datetime(
                                2020,
                                8,
                                9,
                                7,
                                19)),
                            arrival_delay=datetime.timedelta(seconds=60),
                            arrival_platform=None,
                            departure=None,
                            departure_delay=None,
                            departure_platform=None
                        )
                    ],
                    remarks=[
                        Remark(
                            remark_type='A',
                            code='PB',
                            subject=None,
                            text='Pflicht zur Bedeckung von Mund und Nase',
                            priority=200,
                            trip_id=None
                        ),
                        Remark(
                            remark_type='A',
                            code='FK',
                            subject=None,
                            text='Fahrradmitnahme begrenzt möglich',
                            priority=260,
                            trip_id=None
                        ),
                        Remark(
                            remark_type='A',
                            code='NW',
                            subject=None,
                            text='Linie der NordWestBahn, Info unter 01806 600161',
                            priority=899,
                            trip_id=None
                        )
                    ]
                )
            ]
        )
    ]
    assert VSNProfile().parse_journeys_request(hafas_response) == correct_journeys



import abc
import datetime
from typing import Dict, List

from pyhafas.types.fptf import Journey, Station, Leg
from pyhafas.types.hafas_response import HafasResponse


class JourneysRequestInterface(abc.ABC):
    @abc.abstractmethod
    def format_journeys_request(
            self,
            origin: Station,
            destination: Station,
            via: List[Station],
            date: datetime.datetime,
            min_change_time: int,
            max_changes: int,
            products: Dict[str, bool],
            max_journeys: int
    ) -> dict:
        """
        Creates the HaFAS request body for a journeys request

        :param origin: Origin station
        :param destination: Destionation station
        :param via: Via stations, maybe empty list)
        :param date: Date and time to search journeys for
        :param min_change_time: Minimum transfer/change time at each station
        :param max_changes: Maximum number of changes
        :param products: Allowed products (a product is a mean of transport like ICE,IC)
        :param max_journeys: Maximum number of returned journeys
        :return: Request body for HaFAS
        """
        pass

    @abc.abstractmethod
    def format_search_from_leg_request(
            self,
            origin: Leg,
            destination: Station,
            via: List[Station],
            min_change_time: int,
            max_changes: int,
            products: Dict[str, bool],
    ) -> dict:
        """
        Creates the HaFAS request body for a search from leg request

        :param origin: Origin leg
        :param destination: Destionation station
        :param via: Via stations, maybe empty list)
        :param min_change_time: Minimum transfer/change time at each station
        :param max_changes: Maximum number of changes
        :param products: Allowed products (a product is a mean of transport like ICE,IC)
        :return: Request body for HaFAS
        """
        pass

    @abc.abstractmethod
    def parse_journeys_request(self, data: HafasResponse) -> List[Journey]:
        """
        Parses the HaFAS response for a journeys request

        :param data: Formatted HaFAS response
        :return: List of Journey objects
        """
        pass



from Crypto.Cipher import AES
import base64

aid = "rGhXPq+xAlvJd8T8cMnojdD0IoaOY53X7DPAbcXYe5g=" # from res/raw/haf_config.properties; HCI_CHECKSUM
key = bytes([97, 72, 54, 70, 56, 122, 82, 117, 105, 66, 110, 109, 51, 51, 102, 85]) # from de/hafas/g/a/b.java of DBNavigator; probably static

unpad = lambda s : s[:-ord(s[len(s)-1:])] # http://stackoverflow.com/a/12525165/3890934
enc = base64.b64decode(aid)
iv = bytes([00]*16)
cipher = AES.new(key, AES.MODE_CBC, iv)
salt = unpad(cipher.decrypt(enc).decode("utf-8"))

print("Salt:", salt)



import abc
from typing import List

from pyhafas.types.fptf import Station
from pyhafas.types.hafas_response import HafasResponse


class LocationRequestInterface(abc.ABC):
    @abc.abstractmethod
    def format_location_request(self, term: str, rtype: str = 'S'):
        """
        Creates the HaFAS request body for a location search request.

        :param term: Search term
        :param rtype: Result types. One of ['S' for stations, 'ALL' for addresses and stations]
        :return: Request body for HaFAS
        """
        pass

    @abc.abstractmethod
    def parse_location_request(self, data: HafasResponse) -> List[Station]:
        """
        Parses the HaFAS response for a location request

        :param data: Formatted HaFAS response
        :return: List of Station objects
        """
        pass



from pyhafas import HafasClient
from pyhafas.profile import VVVProfile
from pyhafas.types.fptf import Journey


def test_vvv_journey_request():
    client = HafasClient(VVVProfile())
    journey = client.journey(
        journey='¶HKI¶G@F$A=2@O=Adlergasse, 6850 Dornbirn@X=9733124@Y=47401432@b=980000657@a=128@$A=1@O=Dornbirn Bäumlegasse@L=480150302@a=128@$202311010129$202311010138')
    assert isinstance(journey, Journey)



from pyhafas import HafasClient
from pyhafas.profile import VVVProfile


def test_vvv_locations_request():
    client = HafasClient(VVVProfile())
    locations = client.locations(term="Rankweil Bifangstr.")
    assert len(locations) >= 1



import abc
import datetime
from typing import Dict, List, Optional

from pyhafas.types.fptf import Station, StationBoardLeg
from pyhafas.types.hafas_response import HafasResponse
from pyhafas.types.station_board_request import StationBoardRequestType


class StationBoardRequestInterface(abc.ABC):
    def format_station_board_request(
            self,
            station: Station,
            request_type: StationBoardRequestType,
            date: datetime.datetime,
            max_trips: int,
            duration: int,
            products: Dict[str, bool],
            direction: Optional[Station]
    ) -> dict:
        """
        Creates the HaFAS request for Station Board (departure/arrival)

        :param station: Station to get departures/arrivals for
        :param request_type: ARRIVAL or DEPARTURE
        :param date: Date and time to get departures/arrival for
        :param max_trips: Maximum number of trips that can be returned
        :param products: Allowed products (a product is a mean of transport like ICE,IC)
        :param duration: Time in which trips are searched
        :param direction: Direction (end) station of the train. If none, filter will not be applied
        :return: Request body for HaFAS
        """
        pass

    def parse_station_board_request(
            self,
            data: HafasResponse,
            departure_arrival_prefix: str) -> List[StationBoardLeg]:
        """
        Parses the HaFAS data for a station board request

        :param data: Formatted HaFAS response
        :param departure_arrival_prefix: Prefix for specifying whether its for arrival or departure
        :return: List of StationBoardLeg objects
        """
        pass



import datetime
import os

from pyhafas.profile.kvb import KVBProfile
from pyhafas.types.fptf import Station, StationBoardLeg

from tests.types import PyTestHafasResponse


def test_kvb_departures_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/departures_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_station_board_legs = [StationBoardLeg(
        id='1|119823|0|1|24052023',
        name='609',
        direction='Bonn Brüser Berg Hardthöhe/Südwache',
        station=Station(
            id='300068712',
            lid='A=1@O=Bonn Hbf@X=7099420@Y=50731810@U=1@L=300068712@',
            name='Bonn Hbf',
            latitude=50.731810,
            longitude=7.099420),
        date_time=KVBProfile().timezone.localize(datetime.datetime(2023, 5, 24, 15, 57)),
        cancelled=False,
        delay=datetime.timedelta(
            seconds=600),
        platform='D2')]
    assert KVBProfile().parse_station_board_request(
        hafas_response, "d") == correct_station_board_legs



import pytz

from pyhafas.profile import BaseProfile
from pyhafas.profile.vsn.requests.journey import VSNJourneyRequest


class VSNProfile(VSNJourneyRequest, BaseProfile):
    """
    Profile for the HaFAS of "Verkehrsverbund Süd-Niedersachsen" (VSN) - local transportation provider
    """
    baseUrl = "https://fahrplaner.vsninfo.de/hafas/mgate.exe"
    defaultUserAgent = "vsn/5.3.1 (iPad; iOS 13.3; Scale/2.00)"

    salt = 'SP31mBufSyCLmNxp'
    addMicMac = True

    locale = 'de-DE'
    timezone = pytz.timezone('Europe/Berlin')

    requestBody = {
        'client': {
            'id': 'VSN',
            'v': '5030100',
            'type': 'IPA',
            'name': 'vsn',
            'os': 'iOS 13.3'
        },
        'ver': '1.24',
        'lang': 'de',
        'auth': {
            'type': 'AID',
            'aid': 'Mpf5UPC0DmzV8jkg'
        }
    }

    availableProducts = {
        'long_distance_express': [1],  # ICE
        'long_distance': [2],  # IC/EC/CNL
        'regional_express': [4],  # RE/IR
        'regional': [8],  # NV
        'suburban': [16],  # S
        'bus': [32],  # BUS
        'ferry': [64],  # F
        'subway': [128],  # U
        'tram': [256],  # T
        'anruf_sammel_taxi': [512]  # Group Taxi
    }

    defaultProducts = [
        'long_distance_express',
        'long_distance',
        'regional_express',
        'regional',
        'suburban',
        'bus',
        'ferry',
        'subway',
        'tram',
        'anruf_sammel_taxi'
    ]



from pyhafas.profile.base import BaseFormatProductsFilterHelper


class ProductsFilterTestProfile(BaseFormatProductsFilterHelper):
    availableProducts = {
        'long_distance_express': [1, 2],
        'long_distance': [4],
        'regional_express': [8, 16],
    }

    defaultProducts = [
        'long_distance_express',
        'long_distance',
    ]


def test_base_products_filter_without_customization():
    products_filter = ProductsFilterTestProfile().format_products_filter({})
    assert products_filter == {
        'type': 'PROD',
        'mode': 'INC',
        'value': "7"
    }


def test_base_products_filter_with_customization():
    products_filter = ProductsFilterTestProfile().format_products_filter({
        "long_distance_express": False,
        "regional_express": True
    })
    assert products_filter == {
        'type': 'PROD',
        'mode': 'INC',
        'value': "28"
    }



from typing import List

from pyhafas.profile.base import BaseJourneysRequest
from pyhafas.profile.interfaces.requests.journeys import JourneysRequestInterface
from pyhafas.types.fptf import Journey
from pyhafas.types.hafas_response import HafasResponse


class VVVJourneysRequest(BaseJourneysRequest, JourneysRequestInterface):
    def parse_journeys_request(self, data: HafasResponse) -> List[Journey]:
        journeys = []

        for jny in data.res['outConL']:
            date = self.parse_date(jny['date'])

            # skip all 'TRSF' type journeys (propably better handling should be implemented)
            jny['secL'] = [s for s in jny['secL'] if s['type'] != 'TRSF']

            journeys.append(
                Journey(
                    jny['recon']['ctx'], date=date, duration=self.parse_timedelta(
                        jny['dur']), legs=self.parse_legs(
                        jny, data.common, date)))
        return journeys


import datetime
import os

from pyhafas.profile import DBProfile
from pyhafas.types.fptf import Station, StationBoardLeg

from tests.types import PyTestHafasResponse


def test_db_departures_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/departures_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_station_board_legs = [StationBoardLeg(
        id='1|200921|0|80|5082020',
        name='IC 2055',
        direction='Stralsund Hbf',
        station=Station(
            id='8098160',
            lid='A=1@O=Berlin Hbf (tief)@X=13369549@Y=52525589@U=80@L=8098160@',
            name='Berlin Hbf (tief)',
            latitude=52.525589,
            longitude=13.369549),
        date_time=DBProfile().timezone.localize(datetime.datetime(2020, 8, 5, 18, 16)),
        cancelled=False,
        delay=datetime.timedelta(
            seconds=0),
        platform='6')]
    assert DBProfile().parse_station_board_request(
        hafas_response, "d") == correct_station_board_legs



import abc
from typing import Tuple

from pyhafas.types.hafas_response import HafasResponse


class RequestHelperInterface(abc.ABC):
    @abc.abstractmethod
    def calculate_checksum(self, data: str) -> str:
        """
        Calculates the checksum of the request (required for most profiles)

        :param data: Complete body as string
        :return: Checksum for the request
        """
        pass

    @abc.abstractmethod
    def calculate_mic_mac(self, data: str) -> Tuple[str, str]:
        """
        Calculates the mic-mac for the request (required for some profiles)

        :param data: Complete body as string
        :return: Mic and mac to be sent to HaFAS
        """
        pass

    @abc.abstractmethod
    def url_formatter(self, data: str) -> str:
        """
        Formats the URL for HaFAS (adds the checksum or mic-mac)

        :param data: Complete body as string
        :return: Request-URL (maybe with checksum or mic-mac)
        """
        pass

    @abc.abstractmethod
    def request(self, body) -> HafasResponse:
        """
        Sends the request and does a basic parsing of the response and error handling

        :param body: Reqeust body as dict (without the `requestBody` of the profile)
        :return: HafasRespone object or Exception when HaFAS response returns an error
        """
        pass



import datetime
import os

from pyhafas.profile.vsn import VSNProfile
from pyhafas.types.fptf import Station, StationBoardLeg

from tests.types import PyTestHafasResponse


def test_vsn_departures_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/departures_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_station_board_legs = [StationBoardLeg(
        id='1|265947|0|80|1082020',
        name='IC 2315',
        direction='Frankfurt(Main) Hbf',
        station=Station(
            id='8000050',
            lid='A=1@O=Bremen Hbf@X=8813833@Y=53083478@U=80@L=8000050@',
            name='Bremen Hbf',
            latitude=53.083478,
            longitude=8.813833),
        date_time=VSNProfile().timezone.localize(datetime.datetime(2020, 8, 1, 17, 44)),
        cancelled=False,
        delay=datetime.timedelta(
            seconds=3480),
        platform='8')]
    assert VSNProfile().parse_station_board_request(
        hafas_response, "d") == correct_station_board_legs



from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.helper.parse_remark import ParseRemarkHelperInterface
from pyhafas.types.fptf import Remark


class BaseParseRemarkHelper(ParseRemarkHelperInterface):
    def parse_remark(self: ProfileInterface, remark: dict, common: dict) -> Remark:
        """
        Parses Remark HaFAS returns into Remark object

        :param remark: Remark object given back by HaFAS
        :param common: Common object given back by HaFAS
        :return: Parsed Remark object
        """

        rem = Remark(
            remark_type=remark.get('type'),
            code=remark.get('code') if remark.get('code') != "" else None,
            subject=remark.get('txtS') if remark.get('txtS') != "" else None,
            text=remark.get('txtN') if remark.get('txtN') != "" else None,
            priority=remark.get('prio'),
            trip_id=remark.get('jid'),
        )
        # print(rem, remark)
        return rem



import pytz

from pyhafas.profile.base import BaseProfile
from pyhafas.profile.nasa.requests.journeys import NASAJourneysRequest
from pyhafas.profile.nasa.requests.journey import NASAJourneyRequest


class NASAProfile(NASAJourneyRequest, NASAJourneysRequest, BaseProfile):
    """
    Profile of the HaFAS of NASA (Nahverkehr Sachsen-Anhalt).
    """

    baseUrl = "https://reiseauskunft.insa.de/bin/mgate.exe"
    defaultUserAgent = "Dalvik/2.1.0 (Linux; U; Android 11; Pixel 4a Build/RQ2A.210305.006)"

    addMicMac = False
    addChecksum = False

    locale = 'de-DE'
    timezone = pytz.timezone('Europe/Berlin')

    requestBody = {
        "client": {
            "id": "NASA",
            "type": "IPH",
            "name": "nasaProd",
            "v": "4000200",
        },
        "ver": "1.44",
        "lang": "de",
        "auth": {
            "type": "AID",
            "aid": "nasa-apps",
        }
    }

    availableProducts = {
        'long_distance_express': [1],  # ICE
        'long_distance': [2],  # IC/EC/CNL
        'regional': [8],  # RE/RB
        'suburban': [16],  # S
        'bus': [64, 128],  # BUS
        'tram': [32],  # T
        'tourism_train': [256],  # TT
    }

    defaultProducts = [
        'long_distance_express',
        'long_distance',
        'regional',
        'suburban',
        'bus',
        'tram',
        'tourism_train',
    ]



import pytz

from pyhafas.profile.base import BaseProfile


class DBProfile(BaseProfile):
    """
    Profile of the HaFAS of Deutsche Bahn (DB) - German Railway - Regional and long-distance trains throughout Germany
    """
    baseUrl = "https://reiseauskunft.bahn.de/bin/mgate.exe"
    defaultUserAgent = "DB Navigator/19.10.04 (iPhone; iOS 13.1.2; Scale/2.00)"

    salt = 'bdI8UVj40K5fvxwf'
    addChecksum = True

    locale = 'de-DE'
    timezone = pytz.timezone('Europe/Berlin')

    requestBody = {
        'client': {
            'id': 'DB',
            'v': '20100000',
            'type': 'IPH',
            'name': 'DB Navigator'
        },
        'ext': 'DB.R21.12.a',
        'ver': '1.15',
        'auth': {
            'type': 'AID',
            'aid': 'n91dB8Z77MLdoR0K'
        }
    }

    availableProducts = {
        'long_distance_express': [1],  # ICE
        'long_distance': [2],  # IC/EC
        'regional_express': [4],  # RE/IR
        'regional': [8],  # RB
        'suburban': [16],  # S
        'bus': [32],  # BUS
        'ferry': [64],  # F
        'subway': [128],  # U
        'tram': [256],  # T
        'taxi': [512]  # Group Taxi
    }

    defaultProducts = [
        'long_distance_express',
        'long_distance',
        'regional_express',
        'regional',
        'suburban',
        'bus',
        'ferry',
        'subway',
        'tram',
        'taxi'
    ]



import json

import requests

from pyhafas.profile.interfaces.mappings.error_codes import \
    ErrorCodesMappingInterface


class HafasResponse:
    """
    The class HafasResponse handles the general parsing and error-checking of a raw HaFAS response

    :ivar raw_hafas_response: The raw response of HaFAS
    :vartype raw_hafas_response: requests.Response
    :ivar data: json parsed raw response of HaFAS
    :vartype data: dict
    """

    def __init__(self, raw_hafas_response: requests.Response,
                 mapping: ErrorCodesMappingInterface):
        """
        Constructor of class HafasResponse

        :param raw_hafas_response: The raw response of HaFAS
        :param mapping: Error Mapping Enum (key is the HaFAS error code, value the error class)
        """
        data = json.loads(raw_hafas_response.text)
        self.raw_hafas_response = raw_hafas_response
        self.data = data
        self.check_for_errors(mapping)

    def check_for_errors(self, mapping: ErrorCodesMappingInterface):
        """
        Checks if HaFAS response has error messages and handles them

        :param mapping: Error Mapping Enum (key is the HaFAS error code, value the error class)
        """
        error_not_found = False
        if self.data.get('err', "OK") != "OK":
            try:
                raise mapping[self.data['err']].value(
                    self.data.get('errTxt', ''))
            except KeyError:
                error_not_found = True
        if error_not_found:
            raise mapping['default'].value(self.data.get('errTxt', ''))

        if not self.data.get('svcResL', False):
            raise mapping['default'].value("HaFAS response cannot be parsed")

        if self.data['svcResL'][0].get('err', "OK") != "OK":
            try:
                raise mapping[self.data['svcResL'][0]['err']].value(
                    self.data['svcResL'][0].get('errTxt', ''))
            except KeyError:
                error_not_found = True
        if error_not_found:
            raise mapping['default'].value(
                self.data['svcResL'][0].get('errTxt', ''))

    @property
    def common(self):
        """
        Returns the "common" data out of HaFAS data

        :return: dict with "common" data
        """
        return self.data['svcResL'][0]['res']['common']

    @property
    def res(self):
        """
        Returns the "res" data out of HaFAS data

        :return: dict with "res" data
        """
        return self.data['svcResL'][0]['res']



import datetime

from pyhafas import HafasClient
from pyhafas.profile import DBProfile


def test_db_journeys_request():
    client = HafasClient(DBProfile())
    journeys = client.journeys(
        destination="8000135",
        origin="8005556",
        date=datetime.datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys



import datetime

from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.helper.date_time import DateTimeHelperInterface


class BaseDateTimeHelper(DateTimeHelperInterface):
    def parse_datetime(
            self: ProfileInterface,
            time_string: str,
            date: datetime.date) -> datetime.datetime:
        """
        Parses the time format HaFAS returns and combines it with a date

        :param time_string: Time string sent by HaFAS (multiple formats are supported. One example: 234000)
        :param date: Start day of the leg/journey
        :return: Parsed date and time as datetime object
        """
        try:
            hour = int(time_string[-6:-4])
            minute = int(time_string[-4:-2])
            second = int(time_string[-2:])
        except ValueError:
            raise ValueError(
                'Time string "{}" has wrong format'.format(time_string))

        dateOffset = int(time_string[:2]) if len(time_string) > 6 else 0
        return self.timezone.localize(datetime.datetime(
            date.year,
            date.month,
            date.day,
            hour,
            minute,
            second) + datetime.timedelta(days=dateOffset))

    def parse_timedelta(
            self: ProfileInterface,
            time_string: str) -> datetime.timedelta:
        """
        Parses the time HaFAS returns as timedelta object

        Example use case is when HaFAS returns a duration of a leg
        :param time_string: Time string sent by HaFAS (example for format is: 033200)
        :return: Parsed time as timedelta object
        """
        try:
            hours = int(time_string[:2])
            minutes = int(time_string[2:-2])
            seconds = int(time_string[-2:])
        except ValueError:
            raise ValueError(
                'Time string "{}" has wrong format'.format(time_string))

        return datetime.timedelta(
            hours=hours,
            minutes=minutes,
            seconds=seconds)

    def parse_date(self: ProfileInterface, date_string: str) -> datetime.date:
        """
        Parses the date HaFAS returns

        :param date_string: Date sent by HaFAS
        :return: Parsed date object
        """
        dt = datetime.datetime.strptime(date_string, '%Y%m%d')
        return dt.date()

    def transform_datetime_parameter_timezone(
            self: ProfileInterface,
            date_time: datetime.datetime) -> datetime.datetime:
        """
        Transfers datetime parameters incoming by the user to the profile timezone

        :param date_time: datetime parameter incoming by user. Can be timezone aware or unaware
        :return: Timezone aware datetime object in profile timezone
        """
        if date_time.tzinfo is not None and date_time.tzinfo.utcoffset(
                date_time) is not None:
            return date_time.astimezone(self.timezone)
        else:
            return self.timezone.localize(date_time)



import datetime

from pyhafas import HafasClient
from pyhafas.profile import VVVProfile


def test_vvv_arrivals_request():
    client = HafasClient(VVVProfile())
    arrivals = client.arrivals(
        station="480063000",
        date=datetime.datetime.now(),
        max_trips=5
    )
    assert arrivals





from pyhafas import HafasClient
from pyhafas.profile import DBProfile


def test_db_locations_request():
    client = HafasClient(DBProfile())
    locations = client.locations(term="Köln Messe/Deutz")
    assert len(locations) >= 1



from pyhafas import HafasClient
from pyhafas.profile import VSNProfile


def test_vsn_locations_request():
    client = HafasClient(VSNProfile())
    locations = client.locations(term="Göttingen Bahnhof/ZOB")
    assert len(locations) >= 1



from datetime import datetime

from pyhafas import HafasClient
from pyhafas.profile import NASAProfile


# Test arrivals request for NASAProfile of station 'Universitätsbibliothek' with id '7482'

def test_nasa_arrivals_request():
    client = HafasClient(NASAProfile())
    arrivals = client.arrivals(
        station="7482",
        date=datetime.now(),
        max_trips=5
    )
    assert arrivals



import datetime
from enum import Enum
from typing import List, Optional


class Mode(Enum):
    """
    FPTF `Mode` object

    The mode of a `Leg` specifies the general type of transport vehicle (it can also be a walking leg)
    """
    TRAIN = 'train'
    BUS = 'bus'
    WATERCRAFT = 'watercraft'
    TAXI = 'taxi'
    GONDOLA = 'gondola'
    AIRCRAFT = 'aircraft'
    CAR = 'car'
    BICYCLE = 'bicycle'
    WALKING = 'walking'

    def __repr__(self):
        return '<%s.%s>' % (self.__class__.__name__, self.name)


class FPTFObject:
    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.__dict__)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__


class Station(FPTFObject):
    """
    FPTF `Station` object

    A station is a point where vehicles stop. It may be a larger building or just a small stop without special infrastructure.

    :ivar id: ID of the Station. Typically a number but as a string
    :vartype id: str
    :ivar lid: Location ID of the Station (maybe `None`). Long-form, containing multiple fields
    :vartype lid: Optional[str]
    :ivar name: Name of the Station (maybe `None`)
    :vartype name: Optional[str]
    :ivar latitude: Latitude coordinate of the Station (maybe `None`)
    :vartype latitude: Optional[float]
    :ivar longitude: Longitude coordinate of the Station (maybe `None`)
    :vartype longitude: Optional[float]
    """

    def __init__(
            self,
            id: str,
            lid: Optional[str] = None,
            name: Optional[str] = None,
            latitude: Optional[float] = None,
            longitude: Optional[float] = None):
        """
        FPTF `Station` object

        :param id: Internal ID of the station
        :param lid: (optional) Internal Location ID of the station. Defaults to None
        :param name: (optional) Name of the station. Defaults to None
        :param latitude: (optional) Latitude coordinate of the station. Defaults to None
        :param longitude: (optional) Longitude coordinate of the station. Defaults to None
        """
        self.id: str = id
        self.lid: Optional[str] = lid
        self.name: Optional[str] = name
        self.latitude: Optional[float] = latitude
        self.longitude: Optional[float] = longitude


class Remark(FPTFObject):
    """
    A remark is a textual comment/information, usually added to a Stopover or Leg

    :ivar remark_type: Type/Category of the remark. May have a different meaning depending on the network
    :vartype remark_type: Optional[str]
    :ivar code: Code of the remark. May have a different meaning depending on the network
    :vartype code: Optional[str]
    :ivar subject: Subject of the remark
    :vartype subject: Optional[str]
    :ivar text: Actual content of the remark
    :vartype text: Optional[str]
    :ivar priority: Priority of the remark, higher is better
    :vartype priority: Optional[int]
    :ivar trip_id: ID to a Trip added to this remark (e.g. a replacement train)
    :vartype trip_id: Optional[str]
    """
    def __init__(
            self,
            remark_type: Optional[str] = None,
            code: Optional[str] = None,
            subject: Optional[str] = None,
            text: Optional[str] = None,
            priority: Optional[int] = None,
            trip_id: Optional[str] = None):
        """

        :param remark_type: Type/Category of the remark. May have a different meaning depending on the network
        :param code: Code of the remark. May have a different meaning depending on the network
        :param subject: Subject of the remark
        :param text: Actual content of the remark
        :param priority: Priority of the remark, higher is better
        :param trip_id: ID to a Trip added to this remark (e.g. a replacement train)
        """
        self.remark_type: Optional[str] = remark_type
        self.code: Optional[str] = code
        self.subject: Optional[str] = subject
        self.text: Optional[str] = text
        self.priority: Optional[int] = priority
        self.trip_id: Optional[str] = trip_id


class Stopover(FPTFObject):
    """
    FPTF `Stopover` object

    A stopover represents a vehicle stopping at a stop/station at a specific time.

    :ivar stop: Station where the vehicle is stopping
    :vartype stop: Station
    :ivar cancelled: Whether the stop is cancelled
    :vartype cancelled: bool
    :ivar arrival: Planned arrival date and time at the station (maybe `None`)
    :vartype arrival: Optional[datetime.datetime]
    :ivar arrivalDelay: Arrival delay at the station (maybe `None`)
    :vartype arrivalDelay: Optional[datetime.timedelta]
    :ivar arrivalPlatform: Real-time arrival platform at the station (maybe `None`)
    :vartype arrivalPlatform: Optional[str]
    :ivar departure: Planned departure date and time at the station (maybe `None`)
    :vartype departure: Optional[datetime.datetime]
    :ivar departureDelay: Departure delay at the station (maybe `None`)
    :vartype departureDelay: Optional[datetime.timedelta]
    :ivar departurePlatform: Real-time departure platform at the station (maybe `None`)
    :vartype departurePlatform: Optional[str]
    :ivar remarks: (optional) List of remarks
    :vartype remarks: List[Remark]
    """

    def __init__(
            self,
            stop: Station,
            cancelled: bool = False,
            arrival: Optional[datetime.datetime] = None,
            arrival_delay: Optional[datetime.timedelta] = None,
            arrival_platform: Optional[str] = None,
            departure: Optional[datetime.datetime] = None,
            departure_delay: Optional[datetime.timedelta] = None,
            departure_platform: Optional[str] = None,
            remarks: Optional[List[Remark]] = None
    ):
        """

        :param stop: Station where the vehicle is stopping
        :param cancelled: (optional) Whether the stop is cancelled. Defaults to `False`
        :param arrival: (optional) Planned arrival date and time at the station. Defaults to `None`
        :param arrival_delay: (optional) Arrival delay at the station. Defaults to `None`
        :param arrival_platform: (optional) Real-time arrival platform at the station. Defaults to `None`
        :param departure: (optional) Planned departure date and time at the station. Defaults to `None`
        :param departure_delay: (optional) Departure delay at the station. Defaults to `None`
        :param departure_platform: (optional) Real-time departure platform at the station. Defaults to `None`
        :param remarks: (optional) List of remarks. Defaults to `[]`
        """
        self.stop: Station = stop
        self.cancelled: bool = cancelled
        self.arrival: Optional[datetime.datetime] = arrival
        self.arrivalDelay: Optional[datetime.timedelta] = arrival_delay
        self.arrivalPlatform: Optional[str] = arrival_platform
        self.departure: Optional[datetime.datetime] = departure
        self.departureDelay: Optional[datetime.timedelta] = departure_delay
        self.departurePlatform: Optional[str] = departure_platform
        if remarks is None:
            remarks = []
        self.remarks: List[Remark] = remarks


class Leg(FPTFObject):
    """
    FPTF `Leg` object

    A leg or also named trip is most times part of a journey and defines a journey with only one specific vehicle from A to B.

    :ivar id: ID of the Leg
    :vartype id: str
    :ivar origin: FPTF `Station` object of the origin station
    :vartype origin: Station
    :ivar destination: FPTF `Station` object of the destination station
    :vartype destination: Station
    :ivar departure: Planned Date and Time of the departure
    :vartype departure: datetime.datetime
    :ivar arrival: Planned Date and Time of the arrival
    :vartype arrival: datetime.datetime
    :ivar mode: Type of transport vehicle - Must be a part of the FPTF `Mode` enum. Defaults to `Mode.TRAIN`
    :vartype mode: Mode
    :ivar name: Name of the trip (e.g. ICE 123) (maybe `None`)
    :vartype name: Optional[str]
    :ivar cancelled: Whether the trip is completely cancelled (not only some stops)
    :vartype cancelled: bool
    :ivar distance: Distance of the walk trip in metres. Only set if `mode` is `Mode.WALKING` otherwise None
    :vartype distance: Optional[int]
    :ivar departureDelay: Delay at the departure station (maybe `None`)
    :vartype departureDelay: Optional[datetime.timedelta]
    :ivar departurePlatform: Real-time platform at the departure station (maybe `None`)
    :vartype departurePlatform: Optional[str]
    :ivar arrivalDelay: Delay at the arrival station (maybe `None`)
    :vartype arrivalDelay: Optional[datetime.timedelta]
    :ivar arrivalPlatform: Real-time platform at the arrival station (maybe `None`)
    :vartype arrivalPlatform: Optional[str]
    :ivar stopovers: List of FPTF `Stopover` objects (maybe `None`)
    :vartype stopovers: Optional[List[Stopover]]
    :ivar remarks: (optional) List of remarks
    :vartype remarks: List[Remark]
    """

    def __init__(
            self,
            id: str,
            origin: Station,
            destination: Station,
            departure: datetime.datetime,
            arrival: datetime.datetime,
            mode: Mode = Mode.TRAIN,
            name: Optional[str] = None,
            cancelled: bool = False,
            distance: Optional[int] = None,
            departure_delay: Optional[datetime.timedelta] = None,
            departure_platform: Optional[str] = None,
            arrival_delay: Optional[datetime.timedelta] = None,
            arrival_platform: Optional[str] = None,
            stopovers: Optional[List[Stopover]] = None,
            remarks: Optional[List[Remark]] = None
    ):
        """
        FPTF `Leg` object

        :param id: Internal ID of the station
        :param origin: FPTF `Station` object of the origin station
        :param destination: FPTF `Station` object of the destination station
        :param departure: Planned date and Time of the departure
        :param arrival: Planned date and Time of the arrival
        :param mode: (optional) Type of transport vehicle - Must be a part of the FPTF `Mode` enum. Defaults to `Mode.TRAIN`
        :param name: (optional) Name of the trip (e.g. ICE 123). Defaults to None
        :param cancelled: (optional) Whether the trip is cancelled. Defaults to False
        :param distance: (optional) Distance of the walk trip in meters. Defaults to None
        :param departure_delay: (optional) Delay at the departure station. Defaults to None
        :param departure_platform: (optional) Real-time platform at the departure station. Defaults to None
        :param arrival_delay: (optional) Delay at the arrival station. Defaults to None
        :param arrival_platform: (optional) Platform at the arrival station. Defaults to None
        :param stopovers: (optional) List of FPTF `Stopover` objects. Defaults to None
        :param remarks: (optional) List of remarks. Defaults to `[]`
        """
        # Mandatory Variables
        self.id = id
        self.origin: Station = origin
        self.destination: Station = destination
        self.departure: datetime.datetime = departure
        self.arrival: datetime.datetime = arrival

        # Optional Variables
        self.mode: Mode = mode
        self.name: Optional[str] = name
        self.cancelled: bool = cancelled
        self.distance: Optional[int] = distance
        self.departureDelay: Optional[datetime.timedelta] = departure_delay
        self.departurePlatform: Optional[str] = departure_platform
        self.arrivalDelay: Optional[datetime.timedelta] = arrival_delay
        self.arrivalPlatform: Optional[str] = arrival_platform
        self.stopovers: Optional[List[Stopover]] = stopovers
        if remarks is None:
            remarks = []
        self.remarks: List[Remark] = remarks


class Journey(FPTFObject):
    """
    FPTF `Journey` object

    A journey is a computed set of directions to get from A to B at a specific time. It would typically be the result of a route planning algorithm.

    :ivar id: ID of the Journey
    :vartype id: str
    :ivar date: Starting date of the journey (maybe `None`)
    :vartype date: Optional[datetime.date]
    :ivar duration: Duration of the complete journey (maybe `None`)
    :vartype duration: Optional[datetime.timedelta]
    :ivar legs: Longitude coordinate of the Station (maybe `None`)
    :vartype legs: Optional[List[Leg]]
    """

    def __init__(
            self,
            id: str,
            date: Optional[datetime.date] = None,
            duration: Optional[datetime.timedelta] = None,
            legs: Optional[List[Leg]] = None):
        """
        FPTF `Journey` object

        :param id: Internal ID of the station
        :param date: (optional) Name of the station
        :param duration: (optional) Latitude coordinate of the station. Defaults to None
        :param legs: (optional) Longitude coordinate of the station. Defaults to None
        """
        self.id: str = id
        self.date: Optional[datetime.date] = date
        self.duration: Optional[datetime.timedelta] = duration
        self.legs: Optional[List[Leg]] = legs


class StationBoardLeg(FPTFObject):
    """
    `StationBoardLeg` object

    Returned at Station Board-Requests. This requests do not have enough information for a FPTF `Leg` object.
    With the ID a `trip` request can be made to get detailed information about the trip

    :ivar id: ID of the Leg
    :vartype id: str
    :ivar name: Name of the trip (e.g. ICE 123)
    :vartype name: str
    :ivar direction: Direction text of the trip (e.g. Berlin Central Station)
    :vartype direction: str
    :ivar station: FPTF `Station` object of the departing/arriving station
    :vartype station: Station
    :ivar date_time: Planned Date and Time of the departure/arrival
    :vartype date_time: datetime.datetime
    :ivar cancelled: Whether the stop or trip cancelled
    :vartype cancelled: bool
    :ivar delay: Delay at the departure station (maybe `None`)
    :vartype delay: Optional[datetime.timedelta]
    :ivar platform: Real-time platform at the station (maybe `None`)
    :vartype platform: Optional[str]
    """

    def __init__(
            self,
            id: str,
            name: str,
            direction: str,
            station: Station,
            date_time: datetime.datetime,
            cancelled: bool,
            delay: Optional[datetime.timedelta] = None,
            platform: Optional[str] = None
    ):
        """
        `StationBoardLeg` object

        :param id: ID of the Leg
        :param name: Name of the trip (e.g. ICE 123)
        :param direction: Direction text of the trip (e.g. Berlin Central Station)
        :param station: FPTF `Station` object of the departing/arriving station
        :param date_time: Planned Date and Time of the departure/arrival
        :param cancelled: Whether the stop or trip cancelled
        :param delay: (optional) Delay at the departure station. Defaults to `None`
        :param platform: (optional) Real-time platform at the station. Defaults to `None`
        """
        self.id: str = id
        self.name: str = name
        self.direction: str = direction
        self.station: Station = station
        self.dateTime: datetime.datetime = date_time
        self.cancelled: bool = cancelled
        self.delay: Optional[datetime.timedelta] = delay
        self.platform: Optional[str] = platform



from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.requests.journeys import JourneysRequestInterface
from pyhafas.profile.base import BaseJourneysRequest
from pyhafas.types.hafas_response import HafasResponse


class NVVJourneysRequest(BaseJourneysRequest, JourneysRequestInterface):
    def parse_journeys_request(self:ProfileInterface, data: HafasResponse):
        for jny in data.res["outConL"]:
            jny["ctxRecon"] = jny["recon"]["ctx"]

        return super().parse_journeys_request(data)
        


from pyhafas import HafasClient
from pyhafas.profile import VSNProfile
from pyhafas.types.fptf import Journey


def test_vsn_journey_request():
    client = HafasClient(VSNProfile())
    journey = client.journey(
        journey="¶HKI¶T$A=1@O=Lenglern@L=8003644@a=128@$A=1@O=Göttingen@L=4950128@a=128@$202312232144$202312232154$ RB85$$1$$$$$$¶KC¶#VE#2#CF#100#CA#0#CM#0#SICT#0#AM#16433#AM2#0#RT#15#¶KRCC¶#VE#1#",
    )
    assert isinstance(journey, Journey)



import datetime

from pyhafas import HafasClient
from pyhafas.profile import VSNProfile


def test_vsn_journeys_request():
    client = HafasClient(VSNProfile())
    journeys = client.journeys(
        destination="009033817",
        origin="009054997",
        date=datetime.datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys



from pyhafas import HafasClient
from pyhafas.profile import NASAProfile
from pyhafas.types.fptf import Journey


# Test journey request with NASA profile
def test_nasa_journey_request():
    client = HafasClient(NASAProfile())
    journey = client.journey(
        journey="¶HKI¶T$A=1@O=Halle (Saale) Hbf@L=8010159@a=128@$A=1@O=Leipzig Hbf (tief)@L=8098205@a=128@$202312232115$202312232138$ S5X$$1$$$$$$¶KC¶#VE#2#CF#100#CA#0#CM#0#SICT#0#AM#49#AM2#0#RT#15#¶KCC¶I1ZFIzEjRVJHIzEjSElOIzAjRUNLIzE1Njc1fDE1Njc1fDE1Njk4fDE1Njk4fDB8MHw1fDE1Njc1fDF8MHwyfDB8MHwtMjE0NzQ4MzY0OCNHQU0jMjMxMjIzMjExNSMKWiNWTiMxI1NUIzE3MDMyNDY4OTcjUEkjMCNaSSM2MDA0NSNUQSMwI0RBIzIzMTIyMyMxUyM4MDEwMTU5IzFUIzIxMTUjTFMjODAxMDM5NyNMVCMyMjU3I1BVIzgwI1JUIzEjQ0EjcyNaRSNTNVgjWkIjICAgICBTNVgjUEMjNCNGUiM4MDEwMTU5I0ZUIzIxMTUjVE8jODA5ODIwNSNUVCMyMTM4Iw==¶KRCC¶#VE#1#",
    )
    assert isinstance(journey, Journey)



import json
from hashlib import md5
from typing import Tuple

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from pyhafas.profile import ProfileInterface
from pyhafas.profile.base.mappings.error_codes import BaseErrorCodesMapping
from pyhafas.profile.interfaces.helper.request import RequestHelperInterface
from pyhafas.types.hafas_response import HafasResponse


class BaseRequestHelper(RequestHelperInterface):
    request_session = requests.session()

    def calculate_checksum(self: ProfileInterface, data: str) -> str:
        """
        Calculates the checksum of the request (required for most profiles)

        :param data: Complete body as string
        :return: Checksum for the request
        """
        return md5((data + self.salt).encode('utf-8')).hexdigest()

    def calculate_mic_mac(
            self: ProfileInterface,
            data: str) -> Tuple[str, str]:
        """
        Calculates the mic-mac for the request (required for some profiles)

        :param data: Complete body as string
        :return: Mic and mac to be sent to HaFAS
        """
        mic = md5(data.encode('utf-8')).hexdigest()
        mac = self.calculate_checksum(mic)
        return mic, mac

    def url_formatter(self: ProfileInterface, data: str) -> str:
        """
        Formats the URL for HaFAS (adds the checksum or mic-mac)

        :param data: Complete body as string
        :return: Request-URL (maybe with checksum or mic-mac)
        """
        url = self.baseUrl

        if self.addChecksum or self.addMicMac:
            parameters = []
            if self.addChecksum:
                parameters.append(
                    'checksum={}'.format(
                        self.calculate_checksum(data)))
            if self.addMicMac:
                parameters.append(
                    'mic={}&mac={}'.format(
                        *self.calculate_mic_mac(data)))
            url += '?{}'.format('&'.join(parameters))

        return url

    def activate_retry(self: ProfileInterface, retries: int = 4, backoff_factor: float = 1) -> None:
        self.request_session = requests.Session()

        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
        )

        adapter = HTTPAdapter(max_retries=retry)
        self.request_session.mount("http://", adapter)
        self.request_session.mount("https://", adapter)

    def request(self: ProfileInterface, body) -> HafasResponse:
        """
        Sends the request and does a basic parsing of the response and error handling

        :param body: Reqeust body as dict (without the `requestBody` of the profile)
        :return: HafasRespone object or Exception when HaFAS response returns an error
        """
        data = {
            'svcReqL': [body]
        }
        data.update(self.requestBody)
        data = json.dumps(data)

        res = self.request_session.post(
            self.url_formatter(data),
            data=data,
            headers={
                'User-Agent': self.userAgent,
                'Content-Type': 'application/json'})
        return HafasResponse(res, BaseErrorCodesMapping)



import abc
import datetime


class DateTimeHelperInterface(abc.ABC):
    @abc.abstractmethod
    def parse_datetime(
            self,
            time_string: str,
            date: datetime.date) -> datetime.datetime:
        """
        Parses the time format HaFAS returns and combines it with a date

        :param time_string: Time string sent by HaFAS
        :param date: Start day of the leg/journey
        :return: Parsed date and time as datetime object
        """
        pass

    @abc.abstractmethod
    def parse_timedelta(self, time_string: str) -> datetime.timedelta:
        """
        Parses the time HaFAS returns as timedelta object

        Example use case is when HaFAS returns a duration of a leg
        :param time_string: Time string sent by HaFAS
        :return: Parsed time as timedelta object
        """
        pass

    @abc.abstractmethod
    def parse_date(self, date_string: str) -> datetime.date:
        """
        Parses the date HaFAS returns

        :param date_string: Date returned from HaFAS
        :return: Parsed date object
        """
        pass

    @abc.abstractmethod
    def transform_datetime_parameter_timezone(
            self,
            date_time: datetime.datetime) -> datetime.datetime:
        """
        Transfers datetime parameters incoming by the user to the profile timezone

        :param date_time: datetime parameter incoming by user. Can be timezone aware or unaware
        :return: Timezone aware datetime object in profile timezone
        """
        pass



from pyhafas.profile import ProfileInterface
from pyhafas.profile.base import BaseJourneyRequest
from pyhafas.profile.interfaces.requests.journey import JourneyRequestInterface
from pyhafas.types.fptf import Journey


class VSNJourneyRequest(BaseJourneyRequest, JourneyRequestInterface):
    def format_journey_request(
            self: ProfileInterface,
            journey: Journey) -> dict:
        """
        Creates the HaFAS (VSN-deployment) request for refreshing journey details

        :param journey: Id of the journey (ctxRecon)
        :return: Request for HaFAS (VSN-deployment)
        """
        return {
            'req': {
                'outReconL': [{
                    'ctx': journey.id
                }]
            },
            'meth': 'Reconstruction'
        }



import abc


class FormatProductsFilterHelperInterface(abc.ABC):
    @abc.abstractmethod
    def format_products_filter(self, requested_products: dict) -> dict:
        """
        Create the products filter given to HaFAS

        :param requested_products: Mapping of Products to whether it's enabled or disabled
        :return: value for HaFAS "jnyFltrL" attribute
        """
        pass



import os

from pyhafas.profile import DBProfile
from pyhafas.types.fptf import Station

from tests.types import PyTestHafasResponse


def test_db_locations_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/locations_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_locations = [
        Station(
            id='008000207',
            lid='A=1@O=Köln Hbf@X=6958730@Y=50943029@U=80@L=008000207@B=1@p=1596188796@',
            name='Köln Hbf',
            latitude=50.942823,
            longitude=6.959197
        ), Station(
            id="008096022",
            lid='A=1@O=KÖLN@X=6967206@Y=50941312@U=80@L=008096022@B=1@p=1596188796@',
            name='KÖLN',
            latitude=50.941312,
            longitude=6.967206)
    ]
    assert DBProfile().parse_location_request(hafas_response) == correct_locations



import abc

from pyhafas.types.fptf import Leg
from pyhafas.types.hafas_response import HafasResponse


class TripRequestInterface(abc.ABC):
    @abc.abstractmethod
    def format_trip_request(self, trip_id: str) -> dict:
        """
        Creates the HaFAS request for a trip request

        :param trip_id: Id of the trip/leg
        :return: Request body for HaFAS
        """
        pass

    @abc.abstractmethod
    def parse_trip_request(self, data: HafasResponse) -> Leg:
        """
        Parses the HaFAS data for a trip request

        :param data: Formatted HaFAS response
        :return: Leg objects
        """
        pass



from typing import List

from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces import LocationRequestInterface
from pyhafas.types.fptf import Station
from pyhafas.types.hafas_response import HafasResponse


class BaseLocationRequest(LocationRequestInterface):
    def format_location_request(self: ProfileInterface, term: str, rtype: str = 'S'):
        """
        Creates the HaFAS request body for a location search request.

        :param term: Search term
        :param type: Result types. One of ['S' for stations, 'ALL' for addresses and stations]
        :return: Request body for HaFAS
        """
        return {
            "req": {
                "input": {
                    "field": "S",
                    "loc": {
                        "name": term,
                        "type": rtype
                    }
                }
            },
            "meth": "LocMatch"
        }

    def parse_location_request(
            self: ProfileInterface,
            data: HafasResponse) -> List[Station]:
        """
        Parses the HaFAS response for a location request

        :param data: Formatted HaFAS response
        :return: List of Station objects
        """
        stations = []
        for stn in data.res['match']['locL']:
            try:
                latitude: float = stn['crd']['y'] / 1000000
                longitude: float = stn['crd']['x'] / 1000000
            except KeyError:
                latitude: float = 0
                longitude: float = 0
            stations.append(
                self.parse_lid_to_station(
                    stn['lid'],
                    stn['name'],
                    latitude,
                    longitude))
        return stations



import datetime
import os

from pyhafas.profile import VVVProfile
from pyhafas.types.fptf import Station, StationBoardLeg

from tests.types import PyTestHafasResponse


def test_vvv_departures_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/departures_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_station_board_legs = [StationBoardLeg(
        id='2|#VN#1#ST#1698712882#PI#0#ZI#257543#TA#0#DA#11123#1S#480077901#1T#832#LS#480057101#LT#926#PU#81#RT#1#CA#V00#ZE#177#ZB#     177#PC#6#FR#480077901#FT#832#TO#480057101#TT#926#',
        name='Landbus 177',
        direction='Dornbirn Bahnhof',
        station=Station(
            id='480077901',
            lid='A=1@O=Ebnit Heumöser@X=9742491@Y=47345429@U=81@L=480077901@i=A×at:48:779:0:1@',
            name='Ebnit Heumöser',
            latitude=47.345429,
            longitude=9.742491),
        date_time=VVVProfile().timezone.localize(datetime.datetime(2023, 11, 1, 8, 32)),
        cancelled=False,
        delay=None)]
    assert VVVProfile().parse_station_board_request(
        hafas_response, "d") == correct_station_board_legs



import json


class PyTestHafasResponse:
    def __init__(self, raw_hafas_response: str):
        self.raw_hafas_response = raw_hafas_response
        data = json.loads(raw_hafas_response)
        self.data = data

    @property
    def common(self):
        return self.data['svcResL'][0]['res']['common']

    @property
    def res(self):
        return self.data['svcResL'][0]['res']



import datetime

from pyhafas import HafasClient
from pyhafas.profile import DBProfile, VSNProfile

client = HafasClient(DBProfile(), debug=True)

#print(client.departures(
#    station='8000128',
#    date=datetime.datetime.now(),
#    max_trips=5
#))
#
#print(client.arrivals(
#    station='8005556',
#    date=datetime.datetime.now(),
#    max_trips=5
#))
#print(client.journey('¶HKI¶T$A=1@O=Berlin Jungfernheide@L=8011167@a=128@$A=1@O=Berlin Hbf (tief)@L=8098160@a=128@$202002101544$202002101549$RB 18521$$1$§T$A=1@O=Berlin Hbf (tief)@L=8098160@a=128@$A=1@O=München Hbf@L=8000261@a=128@$202002101605$202002102002$ICE 1007$$1$'))
#print(client.journeys(
#        destination="8010101",
#        origin="8010205",
#        date=datetime.datetime(2023,9,23,16,30),
#        min_change_time=0,
#        max_changes=-1
#    ))
#print(client.locations("Leipzig Hbf"))
#print(client.locations("Erfurt Hbf"))

print(client.trip("1|1664416|0|80|23092023"))

#print('='*20)
#vsn = HafasClient(VSNProfile())
#print(vsn.departures(
#    station='9034033',
#    date=datetime.datetime.now(),
#    max_journeys=5
#))
#


import datetime
from typing import Dict, List, Optional, Union

from pyhafas.profile import ProfileInterface
from pyhafas.types.fptf import Journey, Leg, Station, StationBoardLeg
from pyhafas.types.station_board_request import StationBoardRequestType


class HafasClient:
    """
    The interface between the user's program and pyHaFAS internal code.

    :param profile: `Profile` to be used
    :param ua: (optional, not recommended to change) The user-agent which will be sent to HaFAS. By default "pyhafas", but is often overwritten by profile to emulate the app.
    :param debug: (optional) Whether debug mode should be enabled. Defaults to False.
    """

    def __init__(
            self,
            profile: ProfileInterface,
            ua: str = "pyhafas",
            debug: bool = False):
        self.profile = profile
        self.useragent = ua
        self.debug = debug

    def departures(
            self,
            station: Union[Station, str],
            date: datetime.datetime,
            max_trips: int = -1,
            duration: int = -1,
            products: Dict[str, bool] = {},
            direction: Optional[Union[Station, str]] = None) -> List[StationBoardLeg]:
        """
        Returns departing trips at the specified station

        To get detailed information on the trip use the `trip` method with the id

        :param station: FPTF `Station` object or ID of station
        :param date: Date and Time when to search
        :param max_trips: (optional) Maximum number of trips to be returned. Default is "whatever HaFAS wants"
        :param duration: (optional) Minutes after `date` in which is search is made. Default is "whatever HaFAS wants"
        :param products: (optional) Dict of product name(s) and whether it should be enabled or not. Modifies the default products specified in the profile.
        :param direction: (optional) Direction (end) station of the vehicle. Default is any direction station is allowed
        :return: List of FPTF `StationBoardLeg` objects with departing trips
        """
        if not isinstance(station, Station):
            station = Station(id=station)

        if not isinstance(direction, Station) and direction is not None:
            direction = Station(id=direction)

        date = self.profile.transform_datetime_parameter_timezone(date)

        body = self.profile.format_station_board_request(
            station,
            StationBoardRequestType.DEPARTURE,
            date,
            max_trips,
            duration,
            products,
            direction
        )
        res = self.profile.request(body)

        return self.profile.parse_station_board_request(res, "d")

    def arrivals(
            self,
            station: Union[Station, str],
            date: datetime.datetime,
            max_trips: int = -1,
            duration: int = -1,
            products: Dict[str, bool] = {},
            direction: Optional[Union[Station, str]] = None) -> List[StationBoardLeg]:
        """
        Returns arriving trips at the specified station

        To get detailed information on the trip use the `trip` method with the id

        :param station: FPTF `Station` object or ID of station
        :param date: Date and Time when to search
        :param max_trips: (optional) Maximum number of trips to be returned. Default is "whatever HaFAS wants"
        :param duration: (optional) Minutes after `date` in which is search is made. Default is "whatever HaFAS wants"
        :param products: (optional) Dict of product name(s) and whether it should be enabled or not. Modifies the default products specified in the profile.
        :param direction: (optional) Direction (end) station of the vehicle. Default is any direction station is allowed
        :return: List of FPTF `StationBoardLeg` objects with arriving trips
        """
        if not isinstance(station, Station):
            station = Station(id=station)

        if not isinstance(direction, Station) and direction is not None:
            direction = Station(id=direction)

        date = self.profile.transform_datetime_parameter_timezone(date)

        body = self.profile.format_station_board_request(
            station,
            StationBoardRequestType.ARRIVAL,
            date,
            max_trips,
            duration,
            products,
            direction
        )
        res = self.profile.request(body)

        return self.profile.parse_station_board_request(res, "a")

    def journeys(
            self,
            origin: Union[Station, str],
            destination: Union[Station, str],
            date: datetime.datetime,
            via: List[Union[Station, str]] = [],
            min_change_time: int = 0,
            max_changes: int = -1,
            products: Dict[str, bool] = {},
            max_journeys: int = -1
    ) -> List[Journey]:
        """
        Returns possible journeys between two destinations

        Possible journeys between two destinations are calculated by HaFAS and returned. It's also possible to add multiple via stations.

        :param origin: FPTF `Station` object or ID of origin/starting station
        :param destination: FPTF `Station` object or ID of destination/ending station
        :param date: Date and Time when to search
        :param via: (optional) List of via stations. The route is calculated via all of these stations in the order of the list. The stations have to be a FPTF `Station` object or the ID of the station. The default is no via stations.
        :param min_change_time: (optional) Minimum transfer/change time at each station. Default is the default that HaFAS specifies internal.
        :param max_changes: (optional) Maximum number of changes. Default is unlimited.
        :param products: (optional) Dict of product name(s) and whether it should be enabled or not. Modifies the default products specified in the profile.
        :param max_journeys: (optional) Maximum number of returned journeys. Default is the default that HaFAS specifies internal.
        :return: List of FPTF `Journey` objects
        """
        if not isinstance(origin, Station):
            origin = Station(origin)
        if not isinstance(destination, Station):
            destination = Station(destination)
        for via_station in via:
            if not isinstance(via_station, Station):
                via[via.index(via_station)] = Station(via_station)

        date = self.profile.transform_datetime_parameter_timezone(date)

        body = self.profile.format_journeys_request(
            origin,
            destination,
            via,
            date,
            min_change_time,
            max_changes,
            products,
            max_journeys
        )
        res = self.profile.request(body)

        return self.profile.parse_journeys_request(res)

    def journeys_from_leg(
            self,
            origin: Leg,
            destination: Union[Station, str],
            via: List[Union[Station, str]] = [],
            min_change_time: int = 0,
            max_changes: int = -1,
            products: Dict[str, bool] = {},
    ) -> List[Journey]:
        """
        Returns possible journeys from a leg to a destination

        Possible journeys between two destinations are calculated by HaFAS and returned. It's also possible to add multiple via stations.

        :param origin: FPTF `Leg` object from where to search
        :param destination: FPTF `Station` object or ID of destination/ending station
        :param via: (optional) List of via stations. The route is calculated via all of these stations in the order of the list. The stations have to be a FPTF `Station` object or the ID of the station. The default is no via stations.
        :param min_change_time: (optional) Minimum transfer/change time at each station. Default is the default that HaFAS specifies internal.
        :param max_changes: (optional) Maximum number of changes. Default is unlimited.
        :param products: (optional) Dict of product name(s) and whether it should be enabled or not. Modifies the default products specified in the profile.
        :return: List of FPTF `Journey` objects
        """
        if not isinstance(destination, Station):
            destination = Station(destination)
        for via_station in via:
            if not isinstance(via_station, Station):
                via[via.index(via_station)] = Station(via_station)

        body = self.profile.format_search_from_leg_request(
            origin,
            destination,
            via,
            min_change_time,
            max_changes,
            products
        )
        res = self.profile.request(body)

        return self.profile.parse_journeys_request(res)

    def journey(self, journey: Union[Journey, str]) -> Journey:
        """
        Returns information about a specific journey by its ID

        Useful if you want to refresh the data of the trip, e.g. the real-time data.

        :param journey: FPTF `Journey` object or journey ID
        :return: FPTF `Journey` object with current/updated information
        """
        if not isinstance(journey, Journey):
            journey = Journey(journey)

        body = self.profile.format_journey_request(journey)
        res = self.profile.request(body)

        return self.profile.parse_journey_request(res)

    def locations(self, term: str, rtype: str = 'S') -> List[Station]:
        """
        Returns stations (and addresses) that are searched with the provided term

        The further forward the station is in the list, the higher the similarity to the search term.

        :param term: Search term
        :param rtype: Result types. One of ['S' for stations only, 'ALL' for addresses and stations]
        :return: List of FPTF `Station` objects
        """
        body = self.profile.format_location_request(term, rtype)
        res = self.profile.request(body)

        return self.profile.parse_location_request(res)

    def trip(self, id: str) -> Leg:
        """
        Returns detailed information about a trip based on its ID

        :param id: ID of the trip
        :return: Detailed trip information as FPTF `Leg` object
        """
        body = self.profile.format_trip_request(id)
        res = self.profile.request(body)

        return self.profile.parse_trip_request(res)

    def stop(self, stop):
        """
        Not implemented yet.
        """
        raise NotImplementedError

    def nearby(self, location):
        """
        Not implemented yet.
        """
        raise NotImplementedError

    def radar(self, north, west, south, east):
        """
        Not implemented yet.
        """
        raise NotImplementedError



import datetime
import os

from pyhafas.profile import NVVProfile
from pyhafas.types.fptf import Station, StationBoardLeg

from tests.types import PyTestHafasResponse


def test_nvv_departures_parsing():
    directory = os.path.dirname(os.path.realpath(__file__))
    raw_hafas_json_file = open(directory + "/departures_raw.json", "r")
    hafas_response = PyTestHafasResponse(raw_hafas_json_file.read())
    raw_hafas_json_file.close()
    correct_station_board_legs = [StationBoardLeg(
        id='2|#VN#1#ST#1688407889#PI#0#ZI#16504#TA#2#DA#50723#1S#2200012#1T#808#LS#2203069#LT#854#PU#80#RT#1#CA#T#ZE#5#ZB#Tram 5  #PC#5#FR#2200012#FT#808#TO#2203069#TT#854#',
        name='Tram 5',
        direction='Baunatal',
        station=Station(
            id='2200048',
            lid='A=1@O=Kassel Keilsbergstraße@X=9455942@Y=51279199@U=80@L=2200048@',
            name='Kassel Keilsbergstraße',
            latitude=51.279199,
            longitude=9.455942),
        date_time=NVVProfile().timezone.localize(datetime.datetime(2023, 7, 5, 8, 37)),
        cancelled=False,
        delay=datetime.timedelta(
            seconds=0))]
    assert NVVProfile().parse_station_board_request(
        hafas_response, "d") == correct_station_board_legs



from pyhafas import HafasClient
from pyhafas.profile import KVBProfile
from pyhafas.types.fptf import Journey


def test_kvb_journey_request():
    client = HafasClient(KVBProfile())
    journey = client.journey(
        journey="¶HKI¶T$A=1@O=Köln Hbf@L=300000809@a=128@$A=1@O=Köln Ehrenfeld Bf Ehrenfeld@L=300083507@a=128@$202312232115$202312232120$  RE1RRX$$1$$$$$$¶KC¶#VE#2#CF#100#CA#0#CM#0#SICT#0#AM#49#AM2#0#AT0AN#ZP##RT#8#¶KRCC¶#VE#1#IST#¶PC¶eyJncnBJZCI6IlJRX0NMSUVOVCJ9"
    )
    assert isinstance(journey, Journey)



from datetime import datetime
from pyhafas import HafasClient
from pyhafas.profile import NASAProfile


# Test journeys of NASA profile with a trip from 'Universitätsbibliothek' (id: 7482) to 'Neue Str./Zirkusmuseum' (id: 7411)
def test_nasa_journeys():
    client = HafasClient(profile=NASAProfile())
    journeys = client.journeys(
        origin='7482',
        destination='7411',
        date=datetime.now(),
        min_change_time=0,
        max_changes=-1
    )
    assert journeys


from pyhafas.profile.interfaces.mappings.error_codes import \
    ErrorCodesMappingInterface
from pyhafas.types.exceptions import (AccessDeniedError, AuthenticationError,
                                      GeneralHafasError,
                                      JourneysArrivalDepartureTooNearError,
                                      JourneysTooManyTrainsError,
                                      LocationNotFoundError,
                                      TripDataNotFoundError)


class BaseErrorCodesMapping(ErrorCodesMappingInterface):
    """
    Mapping of the HaFAS error code to the exception class

    `default` defines the error when the error code cannot be found in the mapping
    """
    default = GeneralHafasError
    AUTH = AuthenticationError
    R5000 = AccessDeniedError
    LOCATION = LocationNotFoundError
    H500 = JourneysTooManyTrainsError
    H890 = JourneysArrivalDepartureTooNearError
    SQ005 = TripDataNotFoundError
    TI001 = TripDataNotFoundError



#!/usr/bin/env python3
"""Setup file for the HAFAS client."""
import os
import sys


from setuptools import setup,find_packages

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md'), encoding='utf-8') as readme:
    long_description = readme.read()

if sys.argv[-1] == 'publish':
    os.system('python3 setup.py sdist upload')
    sys.exit()

setup(
    name='pyhafas',
    version='0.5.0',
    description='Python client for HAFAS public transport APIs',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/FahrplanDatenGarten/pyhafas',
    download_url='https://github.com/FahrplanDatenGarten/pyhafas/releases',
    author='Ember Keske, Leona Maroni',
    author_email='dev@n0emis.eu, dev@leona.is',
    license='MIT',
    install_requires=['requests~=2.9',
                      'pytz>=2013.6'],
    packages=find_packages(include=['pyhafas', 'pyhafas.*']),
    zip_safe=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
)



import pytz

from pyhafas.profile.base import BaseProfile
from pyhafas.profile.nvv.requests.journey import NVVJourneyRequest
from pyhafas.profile.nvv.requests.journeys import NVVJourneysRequest


class NVVProfile(NVVJourneyRequest, NVVJourneysRequest, BaseProfile):
    """
    Profile for the HaFAs of "Nordhessischer Verkehrs Verbund"
    (NVV) - local transportation provider
    """
    baseUrl = "https://auskunft.nvv.de/bin/mgate.exe"
    defaultUserAgent = "NVV Mobil/5.3.1 (iPhone; IOS 13.1.2; Scale/2.00)"

    addMicMac = False
    addChecksum = False

    locale = "de-DE"
    timezone = pytz.timezone("Europe/Berlin")

    requestBody = {
        "client": {
            "id": "NVV",
            "type": "WEB",
            "name": "webapp"
        },
        "ver": "1.39",
        "lang": "deu",
        "auth":{
            "type": "AID",
            "aid": "R7aKWQLVBRSoVRtY"
        }
    }

    availableProducts = {
        'long_distance_express': [1],   # ICE
        'long_distance': [2],           # IC/EC
        'regional_express': [4],        # RE/RB
        'tram': [32],                   # Tram
        'bus': [64, 128],               # Bus
        'anruf_sammel_taxi': [512],     # Group Taxi
        'regio_tram': [1024]            # Tram / regional express hybrid
    }

    defaultProducts = [
        'long_distance_express',
        'long_distance',
        'regional_express',
        'bus',
        'tram',
        'anruf_sammel_taxi',
        'regio_tram'
    ]



class ProductNotAvailableError(Exception):
    """Requested Product is not available in profile"""
    pass


class GeneralHafasError(Exception):
    """HaFAS returned an general error"""
    pass


class AuthenticationError(Exception):
    """Authentiction data missing or wrong"""
    pass


class AccessDeniedError(Exception):
    """Access is denied"""
    pass


class LocationNotFoundError(Exception):
    """Location/stop not found"""
    pass


class JourneysTooManyTrainsError(Exception):
    """Journeys search: Too many trains, connection is not complete"""
    pass


class JourneysArrivalDepartureTooNearError(Exception):
    """Journeys search: arrival and departure date are too near"""
    pass


class NoDepartureArrivalDataError(Exception):
    """No departure/arrival data available"""
    pass


class TripDataNotFoundError(Exception):
    """No trips found or trip info not available"""



import pytz

from pyhafas.profile.base import BaseProfile


class RKRPProfile(BaseProfile):
    """
    Profile of the HaFAS of Rejsekort & Rejseplan (RKRP) - Danish {Railway, Bus, Metro, etc.}.
    https://www.rejsekort.dk/da/RKRP
    https://help.rejseplanen.dk/hc/da/articles/214174465-Rejseplanens-API
    """

    # Alternative base URLs (mostly CNAMEs):
    #     rejseplanen.dk
    #     webapp.rejseplanen.dk
    #     www.rejseplanen.dk
    #     rkrp.hafas.cloud
    #     rkrp-fat.hafas.cloud
    baseUrl = "https://www.rejseplanen.dk/bin/iphone.exe"
    defaultUserAgent = "Dalvik/2.1.0 (Linux; U; Android 11; Pixel 4a Build/RQ2A.210305.006)"

    addMicMac = False
    addChecksum = False

    locale = 'da-DK'
    timezone = pytz.timezone('Europe/Copenhagen')

    requestBody = {
        "client": {
            "id": "DK",
            "type": "WEB",
            "name": "rejseplanwebapp",
            "l": "vs_webapp"
        },
        "ext": "DK.11",
        "ver": "1.24",
        "lang": "dan",
        "auth": {
            "type": "AID",
            "aid": "j1sa92pcj72ksh0-web"
        }
    }

    availableProducts = {
        'long_distance_express': [1],  # ICE
        'long_distance': [2],  # IC/EC
        'regional_express': [4],  # RE/IR
        'regional': [8],  # RB
        'suburban': [16],  # S
        'bus': [32],  # BUS
        'ferry': [64],  # F
        'subway': [128],  # U
        'tram': [256],  # T
        'taxi': [512]  # Group Taxi
    }

    defaultProducts = [
        'long_distance_express',
        'long_distance',
        'regional_express',
        'regional',
        'suburban',
        'bus',
        'ferry',
        'subway',
        'tram',
        'taxi'
    ]



import datetime
from typing import List

from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.helper.parse_leg import ParseLegHelperInterface
from pyhafas.types.fptf import Leg, Mode, Stopover


class BaseParseLegHelper(ParseLegHelperInterface):
    def parse_leg(
            self: ProfileInterface,
            journey: dict,
            common: dict,
            departure: dict,
            arrival: dict,
            date: datetime.date,
            jny_type: str = "JNY",
            gis=None) -> Leg:
        """
        Parses Leg HaFAS returns into Leg object

        Different Types of HaFAS responses can be parsed into a leg object with the multiple variables

        :param journey: Journey object given back by HaFAS (Data of the Leg to parse)
        :param common:  Common object given back by HaFAS
        :param departure: Departure object given back by HaFAS
        :param arrival: Arrival object given back by HaFAS
        :param date: Parsed date of Journey (Departing date)
        :param jny_type: HaFAS Journey type
        :param gis: GIS object given back by HaFAS. Currently only used by "WALK" journey type.
        :return: Parsed Leg object
        """
        leg_origin = self.parse_lid_to_station(
            common['locL'][departure['locX']]['lid'])
        leg_destination = self.parse_lid_to_station(
            common['locL'][arrival['locX']]['lid'])
        if jny_type == "WALK":
            return Leg(
                id=gis['ctx'],
                origin=leg_origin,
                destination=leg_destination,
                departure=self.parse_datetime(departure['dTimeS'], date),
                arrival=self.parse_datetime(arrival['aTimeS'], date),
                mode=Mode.WALKING,
                name=None,
                distance=gis.get('dist') if gis is not None else None
            )
        else:
            leg_stopovers: List[Stopover] = []
            if 'stopL' in journey:
                for stopover in journey['stopL']:
                    leg_stopovers.append(
                        Stopover(
                            stop=self.parse_lid_to_station(
                                common['locL'][stopover['locX']]['lid']
                            ),
                            cancelled=bool(
                                stopover.get(
                                    'dCncl',
                                    stopover.get(
                                        'aCncl',
                                        False
                                    ))),
                            departure=self.parse_datetime(
                                stopover.get('dTimeS'),
                                date) if stopover.get('dTimeS') is not None else None,
                            departure_delay=self.parse_datetime(
                                stopover['dTimeR'],
                                date) - self.parse_datetime(
                                stopover['dTimeS'],
                                date) if stopover.get('dTimeR') is not None else None,
                            departure_platform=stopover.get(
                                'dPlatfR',
                                stopover.get('dPlatfS', stopover.get('dPltfR', stopover.get('dPltfS', {})).get('txt'))),
                            arrival=self.parse_datetime(
                                stopover['aTimeS'],
                                date) if stopover.get('aTimeS') is not None else None,
                            arrival_delay=self.parse_datetime(
                                stopover['aTimeR'],
                                date) - self.parse_datetime(
                                stopover['aTimeS'],
                                date) if stopover.get('aTimeR') is not None else None,
                            arrival_platform=stopover.get(
                                'aPlatfR',
                                stopover.get('aPlatfS', stopover.get('aPltfR', stopover.get('aPltfS', {})).get('txt'))),
                            remarks=[self.parse_remark(common['remL'][msg['remX']], common)
                                     for msg in stopover.get('msgL', []) if msg.get('remX') is not None]
                        ))

            return Leg(
                id=journey['jid'],
                name=common['prodL'][journey['prodX']]['name'],
                origin=leg_origin,
                destination=leg_destination,
                cancelled=bool(arrival.get('aCncl', False)),
                departure=self.parse_datetime(
                    departure['dTimeS'],
                    date),
                departure_delay=self.parse_datetime(
                    departure['dTimeR'],
                    date) - self.parse_datetime(
                    departure['dTimeS'],
                    date) if departure.get('dTimeR') is not None else None,
                departure_platform=departure.get(
                    'dPlatfR',
                    departure.get('dPlatfS', departure.get('dPltfR', departure.get('dPltfS', {})).get('txt'))),
                arrival=self.parse_datetime(
                    arrival['aTimeS'],
                    date),
                arrival_delay=self.parse_datetime(
                    arrival['aTimeR'],
                    date) - self.parse_datetime(
                    arrival['aTimeS'],
                    date) if arrival.get('aTimeR') is not None else None,
                arrival_platform=arrival.get(
                    'aPlatfR',
                    arrival.get('aPlatfS', arrival.get('aPltfR', arrival.get('aPltfS', {})).get('txt'))),
                stopovers=leg_stopovers,
                remarks=[self.parse_remark(common['remL'][msg['remX']], common)
                         for msg in journey.get('msgL', {}) if msg.get('remX') is not None])

    def parse_legs(
            self: ProfileInterface,
            jny: dict,
            common: dict,
            date: datetime.date) -> List[Leg]:
        """
        Parses Legs (when multiple available)

        :param jny: Journeys object returned by HaFAS (contains secL list)
        :param common: Common object returned by HaFAS
        :param date: Parsed date of Journey (Departing date)
        :return: Parsed List of Leg objects
        """
        legs: List[Leg] = []

        for leg in jny['secL']:
            legs.append(
                self.parse_leg(
                    leg.get(
                        'jny',
                        None),
                    common,
                    leg['dep'],
                    leg['arr'],
                    date,
                    leg['type'],
                    leg.get('gis')))

        return legs



from pyhafas import HafasClient
from pyhafas.profile import NVVProfile


def test_kvb_locations_request():
    client = HafasClient(NVVProfile())
    locations = client.locations(term="Kassel Hbf")
    assert len(locations) >= 1



import datetime
from typing import Dict, List, Optional

from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.requests.station_board import \
    StationBoardRequestInterface
from pyhafas.types.fptf import Station, StationBoardLeg
from pyhafas.types.hafas_response import HafasResponse
from pyhafas.types.station_board_request import StationBoardRequestType


class BaseStationBoardRequest(StationBoardRequestInterface):
    def format_station_board_request(
            self: ProfileInterface,
            station: Station,
            request_type: StationBoardRequestType,
            date: datetime.datetime,
            max_trips: int,
            duration: int,
            products: Dict[str, bool],
            direction: Optional[Station]
    ) -> dict:
        """
        Creates the HaFAS request for a station board request (departure/arrival)

        :param station: Station to get departures/arrivals for
        :param request_type: ARRIVAL or DEPARTURE
        :param date: Date and time to get departures/arrival for
        :param max_trips: Maximum number of trips that can be returned
        :param products: Allowed products (e.g. ICE,IC)
        :param duration: Time in which trips are searched
        :param direction: Direction (end) station of the train. If none, filter will not be applied
        :return: Request body for HaFAS
        """
        # TODO: More options
        return {
            'req': {
                'type': request_type.value,
                'stbLoc': {
                    'lid': 'A=1@L={}@'.format(station.id)
                },
                'dirLoc': {
                    'lid': 'A=1@L={}@'.format(direction.id)
                } if direction is not None else None,
                'maxJny': max_trips,
                'date': date.strftime("%Y%m%d"),
                'time': date.strftime("%H%M%S"),
                'dur': duration,
                'jnyFltrL': [
                    self.format_products_filter(products)
                ],
            },
            'meth': 'StationBoard'
        }

    def parse_station_board_request(
            self: ProfileInterface,
            data: HafasResponse,
            departure_arrival_prefix: str) -> List[StationBoardLeg]:
        """
        Parses the HaFAS data for a station board request

        :param data: Formatted HaFAS response
        :param departure_arrival_prefix: Prefix for specifying whether its for arrival or departure (either a for arrival or d for departure)
        :return: List of StationBoardLeg objects
        """
        legs = []
        if not data.res.get('jnyL', False):
            return legs
        else:
            for raw_leg in data.res['jnyL']:
                date = self.parse_date(raw_leg['date'])

                try:
                    platform = raw_leg['stbStop'][departure_arrival_prefix + 'PltfR']['txt'] if \
                        raw_leg['stbStop'].get(departure_arrival_prefix + 'PltfR') is not None else \
                        raw_leg['stbStop'][departure_arrival_prefix + 'PltfS']['txt']
                except KeyError:
                    platform = raw_leg['stbStop'].get(
                        departure_arrival_prefix + 'PlatfR',
                        raw_leg['stbStop'].get(
                            departure_arrival_prefix + 'PlatfS',
                            None))

                legs.append(StationBoardLeg(
                    id=raw_leg['jid'],
                    name=data.common['prodL'][raw_leg['prodX']]['name'],
                    direction=raw_leg['dirTxt'],
                    date_time=self.parse_datetime(
                        raw_leg['stbStop'][departure_arrival_prefix + 'TimeS'],
                        date
                    ),
                    station=self.parse_lid_to_station(data.common['locL'][raw_leg['stbStop']['locX']]['lid']),
                    platform=platform,
                    delay=self.parse_datetime(
                        raw_leg['stbStop'][departure_arrival_prefix + 'TimeR'],
                        date) - self.parse_datetime(
                        raw_leg['stbStop'][departure_arrival_prefix + 'TimeS'],
                        date) if raw_leg['stbStop'].get(departure_arrival_prefix + 'TimeR') is not None else None,
                    cancelled=bool(raw_leg['stbStop'].get(departure_arrival_prefix + 'Cncl', False))
                ))
            return legs



import pytz

from pyhafas.profile import BaseProfile
from pyhafas.profile.kvb.requests.journeys import KVBJourneysRequest
from pyhafas.profile.kvb.requests.journey import KVBJourneyRequest


class KVBProfile(KVBJourneysRequest, KVBJourneyRequest, BaseProfile):
    """
    Profile of the HaFAS of Kölner Verkehrsbetriebe (KVB) - Regional in Cologne
    """
    baseUrl = "https://auskunft.kvb.koeln/gate"

    locale = 'de-DE'
    timezone = pytz.timezone('Europe/Berlin')

    requestBody = {
        'client': {
            'id': 'HAFAS',
            'l': 'vs_webapp',
            'v': '154',
            'type': 'WEB',
            'name': 'webapp'
        },
        'ext': 'DB.R21.12.a',
        'ver': '1.58',
        'lang': 'deu',
        'auth': {
            'type': 'AID',
            'aid': 'Rt6foY5zcTTRXMQs'
        }
    }

    availableProducts = {
        's-bahn': [1],
        'stadtbahn': [2],
        'bus': [8],
        'fernverkehr': [32],
        'regionalverkehr': [64],
        'taxibus': [256]
    }

    defaultProducts = [
        's-bahn',
        'stadtbahn',
        'bus',
        'fernverkehr',
        'regionalverkehr',
        'taxibus',
    ]



import datetime

from pyhafas import HafasClient
from pyhafas.profile import DBProfile
from pyhafas.types.fptf import Journey


def test_db_journey_request():
    client = HafasClient(DBProfile())
    journey = client.journey(
        journey="¶HKI¶T$A=1@O=Siegburg/Bonn@L=8005556@a=128@$A=1@O=Hennef(Sieg)@L=8002753@a=128@$202306101439$202306101445$S     12$$1$$$",
    )
    assert isinstance(journey, Journey)



import datetime

from pyhafas import HafasClient
from pyhafas.profile import DBProfile


def test_db_departures_request():
    client = HafasClient(DBProfile())
    departures = client.departures(
        station="8011160",
        date=datetime.datetime.now(),
        max_trips=2
    )
    assert len(departures) <= 2



import abc

from pyhafas.types.fptf import Journey
from pyhafas.types.hafas_response import HafasResponse


class JourneyRequestInterface(abc.ABC):
    @abc.abstractmethod
    def format_journey_request(self, journey: Journey) -> dict:
        """
        Creates the HaFAS request body for a journey request

        :param journey: Id of the journey (ctxRecon)
        :return: Request body for HaFAS
        """
        pass

    @abc.abstractmethod
    def parse_journey_request(self, data: HafasResponse) -> Journey:
        """
        Parses the HaFAS response for a journey request

        :param data: Formatted HaFAS response
        :return: List of Journey objects
        """
        pass



from pyhafas.profile import ProfileInterface
from pyhafas.profile.interfaces.requests.trip import TripRequestInterface
from pyhafas.types.fptf import Leg
from pyhafas.types.hafas_response import HafasResponse


class BaseTripRequest(TripRequestInterface):
    def format_trip_request(self: ProfileInterface, trip_id: str) -> dict:
        """
        Creates the HaFAS request for a trip request

        :param trip_id: Id of the trip/leg
        :return: Request body for HaFAS
        """
        return {
            'req': {
                'jid': trip_id
            },
            'meth': 'JourneyDetails'
        }

    def parse_trip_request(self: ProfileInterface, data: HafasResponse) -> Leg:
        """
        Parses the HaFAS data for a trip request

        :param data: Formatted HaFAS response
        :return: Leg objects
        """
        return self.parse_leg(
            data.res['journey'],
            data.common,
            data.res['journey']['stopL'][0],
            data.res['journey']['stopL'][-1],
            self.parse_date(data.res['journey']['date'])
        )
